<?php
/**
 * @package    Joomla.Component.Builder
 *
 * @created    4th September, 2022
 * @author     Llewellyn van der Merwe <https://dev.vdm.io>
 * @git        Joomla Component Builder <https://git.vdm.dev/joomla/Component-Builder>
 * @copyright  Copyright (C) 2015 Vast Development Method. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace VDM\Joomla\Componentbuilder;


use VDM\Joomla\Interfaces\TableInterface;
use VDM\Joomla\Abstraction\BaseTable;


/**
 * Componentbuilder Tables
 * 
 * @since 3.2.0
 */
class Table extends BaseTable implements TableInterface
{
	/**
	 * All areas/views/tables with their field details
	 *
	 * @var     array
	 * @since 3.2.0
	 **/
	protected array $tables = [
		'joomla_component' => [
			'system_name' => [
				'name' => 'system_name',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_SYSTEM_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'acfe906b-6e61-4f94-ae66-359e4bc3e4cc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'name_code' => [
				'name' => 'name_code',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_NAME_CODE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'af7b6b08-e8c5-4551-b6e6-cde297e78e40',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'short_description' => [
				'name' => 'short_description',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_SHORT_DESCRIPTION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '46503007-43d9-485f-8cd0-8427a6a8bef2',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'companyname' => [
				'name' => 'companyname',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_COMPANYNAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => 'f8c2c432-b756-4c67-bfa5-0871e58e55e3',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'javascript' => [
				'name' => 'javascript',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_JAVASCRIPT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '3be0d9ca-afd6-47bc-87e5-8cb73e9f5e61',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'css_site' => [
				'name' => 'css_site',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_CSS_SITE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '667a2147-05d1-4138-b666-e8a0ce54ffba',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_helper_site' => [
				'name' => 'php_helper_site',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_PHP_HELPER_SITE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'f4134f92-7dc5-4729-9a16-bfaa51de27f9',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_sales_server' => [
				'name' => 'add_sales_server',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_SALES_SERVER_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '378a9839-97ce-4bdb-8961-a9164738a3bf',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_jcb_powers_path' => [
				'name' => 'add_jcb_powers_path',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_JCB_POWERS_PATH_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'a6c65a43-e305-42c1-bd9d-5db5e4920185',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'debug_linenr' => [
				'name' => 'debug_linenr',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_DEBUG_LINENR_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f5e0c377-6689-4f79-9a35-41a1bfc58ba5',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'mvc_versiondate' => [
				'name' => 'mvc_versiondate',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_MVC_VERSIONDATE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '508b5e78-c91a-4aaa-b587-000759130d4e',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'remove_line_breaks' => [
				'name' => 'remove_line_breaks',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_REMOVE_LINE_BREAKS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '1',
					'GUID' => '2089f202-667a-4fbd-881c-cc0fe1343ce4',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_placeholders' => [
				'name' => 'add_placeholders',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_PLACEHOLDERS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '991ec7f9-257a-486c-9453-c6b1c3fd5bf0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_preflight_update' => [
				'name' => 'php_preflight_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_PHP_PREFLIGHT_UPDATE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '9f9b030b-31ba-4ae9-88da-76efd8a140dc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_DESCRIPTION_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '749a9917-90c3-49c4-9e72-aa33b0683a87',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'sql_uninstall' => [
				'name' => 'sql_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_SQL_UNINSTALL_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'b877b188-693c-43a3-9121-a37a3943d7c1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'author' => [
				'name' => 'author',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_AUTHOR_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'd4a595fb-2264-4e18-8b80-7affee1a6abf',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'email' => [
				'name' => 'email',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_EMAIL_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '2a9d2cda-1be7-4ae7-b672-9139a8a62d14',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'backup_folder_path' => [
				'name' => 'backup_folder_path',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_BACKUP_FOLDER_PATH_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '58525d14-8cbf-49fc-8d8e-50a9ba0e846a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'website' => [
				'name' => 'website',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_WEBSITE_LABEL',
				'type' => 'url',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'CHAR(255)',
					'default' => '',
					'GUID' => '9f474240-eb27-40d7-9512-fbb3acf370d5',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'crowdin_project_identifier' => [
				'name' => 'crowdin_project_identifier',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_CROWDIN_PROJECT_IDENTIFIER_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '9e7c209f-68df-411a-a6a3-591393bc9d62',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'license' => [
				'name' => 'license',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_LICENSE_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'e5346711-63a0-45b8-a093-06486959a1d0',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'created' => [
				'name' => 'created',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_CREATED_LABEL',
				'type' => 'calendar',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'DATETIME',
					'default' => 'CURRENT_TIMESTAMP',
					'GUID' => 'e091bf65-58cb-4bf4-aba3-22015889acc6',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'bom' => [
				'name' => 'bom',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_BOM_LABEL',
				'type' => 'filelist',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => 'b659c01d-accd-4049-bd4c-083000740d5a',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_site_event' => [
				'name' => 'php_site_event',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_PHP_SITE_EVENT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'a052df3d-54b4-4da7-a0f6-f18703d1047b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'image' => [
				'name' => 'image',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_IMAGE_LABEL',
				'type' => 'media',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '4003ab3f-70d8-4092-a11b-b23215844ffa',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'css_admin' => [
				'name' => 'css_admin',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_CSS_ADMIN_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '33c588ca-83b2-419d-b87f-b9f4e5e4ea54',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'copyright' => [
				'name' => 'copyright',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_COPYRIGHT_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'd4241cde-3446-4318-9269-905b7ea929f4',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_postflight_update' => [
				'name' => 'php_postflight_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_PHP_POSTFLIGHT_UPDATE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'ade283c3-937e-4502-a040-c69d6ddb3c50',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_preflight_install' => [
				'name' => 'php_preflight_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_PHP_PREFLIGHT_INSTALL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '3ca01651-a5c3-46c3-b7a8-bfb0d9f6e105',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_method_install' => [
				'name' => 'php_method_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_PHP_METHOD_INSTALL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '8cb26845-73d8-438a-b28a-7a128a63a0d7',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_postflight_install' => [
				'name' => 'php_postflight_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_PHP_POSTFLIGHT_INSTALL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '8d837989-4a21-4943-bb4c-4d4abb9a7a55',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'component_version' => [
				'name' => 'component_version',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_COMPONENT_VERSION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => 'dde96c4d-1528-402a-bdea-96ec481a11ef',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_method_uninstall' => [
				'name' => 'php_method_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_PHP_METHOD_UNINSTALL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e76cf555-112a-4021-9a1e-b4ee52d7be8a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'preferred_joomla_version' => [
				'name' => 'preferred_joomla_version',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_PREFERRED_JOOMLA_VERSION_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(11)',
					'default' => '3',
					'GUID' => 'e30ca545-0ccd-4925-b34d-96bc27d6a27c',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'sql' => [
				'name' => 'sql',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_SQL_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '5eb879c0-f6e3-45b6-b220-cb5131422a83',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_powers' => [
				'name' => 'add_powers',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_POWERS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '1',
					'GUID' => '9ae98a1d-cc61-49d4-8f80-b4e1257fce24',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'addreadme' => [
				'name' => 'addreadme',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADDREADME_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Readme',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '03de1a61-d639-4d48-8ce6-fcf16c8f6270',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'update_server_url' => [
				'name' => 'update_server_url',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_UPDATE_SERVER_URL_LABEL',
				'type' => 'url',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'e6c263b1-da39-4a73-8585-637dc3a5b8eb',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_backup_folder_path' => [
				'name' => 'add_backup_folder_path',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_BACKUP_FOLDER_PATH_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '46bd236f-7d43-4038-ac40-936dfd18e910',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'translation_tool' => [
				'name' => 'translation_tool',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_TRANSLATION_TOOL_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '43d7329c-c5f7-4bde-ab6e-a9a743879b51',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'crowdin_username' => [
				'name' => 'crowdin_username',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_CROWDIN_USERNAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'basic_encryption',
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '441646d1-27c5-4ea9-80d9-a1931de48b26',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'buildcompsql' => [
				'name' => 'buildcompsql',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_BUILDCOMPSQL_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Dynamic Build',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '98ad53ca-0da2-4aab-9a3c-1c7e1ee0a24e',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_site_event' => [
				'name' => 'add_site_event',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_SITE_EVENT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '8b68326a-0e09-4df9-807e-4c32c3c9600a',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_namespace_prefix' => [
				'name' => 'add_namespace_prefix',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_NAMESPACE_PREFIX_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'CHAR(1)',
					'default' => '',
					'GUID' => '50d55392-74b6-43e2-9b16-37c7f95a7402',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_javascript' => [
				'name' => 'add_javascript',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_JAVASCRIPT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '7d31d363-03b6-4ca7-bbf4-df9a879e3006',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'namespace_prefix' => [
				'name' => 'namespace_prefix',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_NAMESPACE_PREFIX_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'b742dcb4-0d29-4a4a-88f9-eca218d79a19',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_css_admin' => [
				'name' => 'add_css_admin',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_CSS_ADMIN_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'c2afae90-c059-4efb-9705-b7cf161b0c2a',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_css_site' => [
				'name' => 'add_css_site',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_CSS_SITE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'c5cbf873-d13a-4930-9c99-c8b1f03e1a06',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_menu_prefix' => [
				'name' => 'add_menu_prefix',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_MENU_PREFIX_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'CHAR(1)',
					'default' => '',
					'GUID' => '0e5f211c-d6d7-417a-af92-f3e956687f56',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'dashboard_type' => [
				'name' => 'dashboard_type',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_DASHBOARD_TYPE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '1',
					'GUID' => '36fc971f-a446-49ae-94d0-2f270e60bd0b',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'menu_prefix' => [
				'name' => 'menu_prefix',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_MENU_PREFIX_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'VARCHAR(100)',
					'default' => '',
					'GUID' => '7971c559-0867-44a8-bfea-291881bad311',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'dashboard' => [
				'name' => 'dashboard',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_DASHBOARD_LABEL',
				'type' => 'dynamicdashboard',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'GUID' => 'b12fadb6-2825-465e-a1e7-1dde507ffde5',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_custom_admin_view',
					'component' => 'com_componentbuilder',
					'entity' => 'custom_admin_view',
					'value' => 'system_name',
					'key' => '',
				],
			],
			'add_php_preflight_install' => [
				'name' => 'add_php_preflight_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_PHP_PREFLIGHT_INSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'd5d03647-f29e-44a4-9e63-5feb9f4adbae',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_preflight_update' => [
				'name' => 'add_php_preflight_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_PHP_PREFLIGHT_UPDATE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '365d08ee-fe6e-47c4-b567-a001212f2cb6',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'toignore' => [
				'name' => 'toignore',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_TOIGNORE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'cce91031-8bf4-435f-8b20-9224f230061a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_postflight_install' => [
				'name' => 'add_php_postflight_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_PHP_POSTFLIGHT_INSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '3c1db8b4-65f2-4f6f-8391-5883f564eabe',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_postflight_update' => [
				'name' => 'add_php_postflight_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_PHP_POSTFLIGHT_UPDATE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '22a668bc-d169-4346-9929-3819673c67ac',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_method_uninstall' => [
				'name' => 'add_php_method_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_PHP_METHOD_UNINSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '70a2d820-64af-4f51-8b1a-522aed9572bf',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'addcontributors' => [
				'name' => 'addcontributors',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADDCONTRIBUTORS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'json',
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '8e416ff9-3bd1-4115-a08b-06594978ace6',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'name' => [
						'name' => 'name',
						'type' => 'text',
						'link' => NULL,
					],
					'title' => [
						'name' => 'title',
						'type' => 'text',
						'link' => NULL,
					],
					'email' => [
						'name' => 'email',
						'type' => 'text',
						'link' => NULL,
					],
					'website' => [
						'name' => 'website',
						'type' => 'url',
						'link' => NULL,
					],
					'use' => [
						'name' => 'use',
						'type' => 'list',
						'link' => NULL,
					],
					'show' => [
						'name' => 'show',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'add_php_method_install' => [
				'name' => 'add_php_method_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_PHP_METHOD_INSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dash & Install',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '9a825342-c423-42d2-85ad-f74ce9625f99',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'emptycontributors' => [
				'name' => 'emptycontributors',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_EMPTYCONTRIBUTORS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '05e16c3f-1f5e-4355-a948-5331546f218c',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_sql' => [
				'name' => 'add_sql',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_SQL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '7c652c71-e00e-45ff-8acb-e1bc0344e4c0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'number' => [
				'name' => 'number',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_NUMBER_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'GUID' => 'c2ce0b7e-e917-40c0-9c63-016425dd11fd',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_sql_uninstall' => [
				'name' => 'add_sql_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_SQL_UNINSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f22fd59e-4d06-4578-b0bb-a624b8efd145',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'assets_table_fix' => [
				'name' => 'assets_table_fix',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ASSETS_TABLE_FIX_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '3',
					'GUID' => '1831e024-aa0c-4fc7-b1e6-77ce4dd4c562',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'readme' => [
				'name' => 'readme',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_README_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Readme',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '87a4b892-c4cb-4032-8511-e18ba8c8c202',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_update_server' => [
				'name' => 'add_update_server',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_UPDATE_SERVER_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'db2835aa-b0c3-4c9a-abe6-5b09568fffda',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'update_server_target' => [
				'name' => 'update_server_target',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_UPDATE_SERVER_TARGET_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'b9f98c3d-a93a-4b19-b89e-4c62781f9b99',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'update_server' => [
				'name' => 'update_server',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_UPDATE_SERVER_LABEL',
				'type' => 'servers',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'GUID' => '33fb33c3-1ff2-4d4c-8fe8-6aaa93c1646a',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_server',
					'component' => 'com_componentbuilder',
					'entity' => 'server',
					'value' => 'name',
					'key' => 'id',
				],
			],
			'creatuserhelper' => [
				'name' => 'creatuserhelper',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_CREATUSERHELPER_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'ec128453-f0a7-4b20-a109-56d85c6ce213',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'sales_server' => [
				'name' => 'sales_server',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_SALES_SERVER_LABEL',
				'type' => 'servers',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'GUID' => '4553e8c2-3446-4e11-a967-0504509087a4',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_server',
					'component' => 'com_componentbuilder',
					'entity' => 'server',
					'value' => 'name',
					'key' => 'id',
				],
			],
			'adduikit' => [
				'name' => 'adduikit',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADDUIKIT_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '627ec119-f50c-4afc-bfe7-8455a7ace163',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'addfootable' => [
				'name' => 'addfootable',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADDFOOTABLE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'ecdc0281-fc1b-4643-b9d0-41bf95674959',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_git_folder_path' => [
				'name' => 'add_git_folder_path',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_GIT_FOLDER_PATH_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'eac7c528-4764-45e8-8304-4b8701e10915',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_email_helper' => [
				'name' => 'add_email_helper',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_EMAIL_HELPER_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'b4013eb9-0716-42a5-a479-6c4b53746cd6',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'git_folder_path' => [
				'name' => 'git_folder_path',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_GIT_FOLDER_PATH_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '7b181242-b4bc-4261-87cb-6210ec8b5af5',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_helper_both' => [
				'name' => 'add_php_helper_both',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_PHP_HELPER_BOTH_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '5dabe624-329e-48d6-82cf-1f417df55467',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'jcb_powers_path' => [
				'name' => 'jcb_powers_path',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_JCB_POWERS_PATH_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '763e5bdf-d4d3-449b-bb36-de2623d81540',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_helper_both' => [
				'name' => 'php_helper_both',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_PHP_HELPER_BOTH_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '16ec7ebd-d07a-4134-8537-d7b65bcd8bac',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_helper_admin' => [
				'name' => 'add_php_helper_admin',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_PHP_HELPER_ADMIN_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'abc5e604-4a2b-4f4c-9add-9e371249adf8',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'crowdin_project_api_key' => [
				'name' => 'crowdin_project_api_key',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_CROWDIN_PROJECT_API_KEY_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'basic_encryption',
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '81cd22bd-ee49-470e-afa1-1f61b302734b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_helper_admin' => [
				'name' => 'php_helper_admin',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_PHP_HELPER_ADMIN_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '16a76125-d200-4bbf-aa00-91da3513a8f8',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'crowdin_account_api_key' => [
				'name' => 'crowdin_account_api_key',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_CROWDIN_ACCOUNT_API_KEY_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'basic_encryption',
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '0708c9c5-d918-43f9-8383-f3d4c7459cf9',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_admin_event' => [
				'name' => 'add_admin_event',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_ADMIN_EVENT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f114341a-d271-4917-a5eb-8d5acdf0232d',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'buildcomp' => [
				'name' => 'buildcomp',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_BUILDCOMP_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Dynamic Build',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '288e00d8-7ea3-4bfc-829c-3ab247ca9718',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_admin_event' => [
				'name' => 'php_admin_event',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_PHP_ADMIN_EVENT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_components',
				'store' => 'base64',
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'a8b0ea74-9315-4f24-be43-836f249645f3',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_helper_site' => [
				'name' => 'add_php_helper_site',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_ADD_PHP_HELPER_SITE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Libs & Helpers',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'ff624860-89db-4465-97f1-962bf09dcc9f',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'modified' => [
				'name' => 'modified',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_MODIFIED_LABEL',
				'type' => 'calendar',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'DATETIME',
					'default' => 'EMPTY',
					'GUID' => '104b94e8-260f-4517-a4f8-80c6b3d4a10a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_COMPONENT_NAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_components',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => 'b607dfed-bb4a-463d-9e14-f1264c67279b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
			'metakey' => [
				'name' => 'metakey',
				'label' => 'Meta Keywords',
				'type' => 'textarea',
				'title' => false,
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'TEXT',
				],
			],
			'metadesc' => [
				'name' => 'metadesc',
				'label' => 'Meta Description',
				'type' => 'textarea',
				'title' => false,
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'TEXT',
				],
			],
			'metadata' => [
				'name' => 'metadata',
				'label' => 'Meta Data',
				'type' => NULL,
				'title' => false,
				'store' => 'json',
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'TEXT',
				],
			],
		],
		'joomla_module' => [
			'system_name' => [
				'name' => 'system_name',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_SYSTEM_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'HTML',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'acfe906b-6e61-4f94-ae66-359e4bc3e4cc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'target' => [
				'name' => 'target',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_TARGET_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'HTML',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '88d58023-7ee7-46dc-a13e-064fbb3d8dc1',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_DESCRIPTION_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'HTML',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '749a9917-90c3-49c4-9e72-aa33b0683a87',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_method_uninstall' => [
				'name' => 'add_php_method_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADD_PHP_METHOD_UNINSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '70a2d820-64af-4f51-8b1a-522aed9572bf',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_postflight_update' => [
				'name' => 'add_php_postflight_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADD_PHP_POSTFLIGHT_UPDATE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '22a668bc-d169-4346-9929-3819673c67ac',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_postflight_install' => [
				'name' => 'add_php_postflight_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADD_PHP_POSTFLIGHT_INSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '3c1db8b4-65f2-4f6f-8391-5883f564eabe',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_preflight_uninstall' => [
				'name' => 'add_php_preflight_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADD_PHP_PREFLIGHT_UNINSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '3bb6d77d-05bf-4447-a6bb-4af197ec9290',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'addreadme' => [
				'name' => 'addreadme',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADDREADME_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Readme',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '03de1a61-d639-4d48-8ce6-fcf16c8f6270',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'default' => [
				'name' => 'default',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_DEFAULT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'HTML',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '6e214a72-6c96-40c0-abf3-5f09116d3018',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'snippet' => [
				'name' => 'snippet',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_SNIPPET_LABEL',
				'type' => 'snippets',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'HTML',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'fc4bc2f3-1a12-4484-88dc-273199fe3b63',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_snippet',
					'component' => 'com_componentbuilder',
					'entity' => 'snippet',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'add_sql' => [
				'name' => 'add_sql',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADD_SQL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '7c652c71-e00e-45ff-8acb-e1bc0344e4c0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'update_server_target' => [
				'name' => 'update_server_target',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_UPDATE_SERVER_TARGET_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'b9f98c3d-a93a-4b19-b89e-4c62781f9b99',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_sql_uninstall' => [
				'name' => 'add_sql_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADD_SQL_UNINSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f22fd59e-4d06-4578-b0bb-a624b8efd145',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'update_server' => [
				'name' => 'update_server',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_UPDATE_SERVER_LABEL',
				'type' => 'servers',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'GUID' => '33fb33c3-1ff2-4d4c-8fe8-6aaa93c1646a',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_server',
					'component' => 'com_componentbuilder',
					'entity' => 'server',
					'value' => 'name',
					'key' => 'id',
				],
			],
			'add_update_server' => [
				'name' => 'add_update_server',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADD_UPDATE_SERVER_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'db2835aa-b0c3-4c9a-abe6-5b09568fffda',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'libraries' => [
				'name' => 'libraries',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_LIBRARIES_LABEL',
				'type' => 'libraries',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'json',
				'tab_name' => 'HTML',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'f17742b9-0096-4b03-b981-f0dcd3f7e8c1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_library',
					'component' => 'com_componentbuilder',
					'entity' => 'library',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'module_version' => [
				'name' => 'module_version',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_MODULE_VERSION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'HTML',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => 'dfb4e6cd-5fb6-4476-9772-f2c8e2143bb6',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'sales_server' => [
				'name' => 'sales_server',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_SALES_SERVER_LABEL',
				'type' => 'servers',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'GUID' => '4553e8c2-3446-4e11-a967-0504509087a4',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_server',
					'component' => 'com_componentbuilder',
					'entity' => 'server',
					'value' => 'name',
					'key' => 'id',
				],
			],
			'custom_get' => [
				'name' => 'custom_get',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_CUSTOM_GET_LABEL',
				'type' => 'customgets',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'json',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'bed7693b-ccd1-4803-b065-5e20fac13881',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_dynamic_get',
					'component' => 'com_componentbuilder',
					'entity' => 'dynamic_get',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'php_preflight_update' => [
				'name' => 'php_preflight_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_PHP_PREFLIGHT_UPDATE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '9f9b030b-31ba-4ae9-88da-76efd8a140dc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_preflight_uninstall' => [
				'name' => 'php_preflight_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_PHP_PREFLIGHT_UNINSTALL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '628147c3-5f93-4a49-a1bc-ac6685e06b64',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'mod_code' => [
				'name' => 'mod_code',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_MOD_CODE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'a6138fdf-ad70-4ec6-aee7-b3e5fda2e2bb',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_postflight_install' => [
				'name' => 'php_postflight_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_PHP_POSTFLIGHT_INSTALL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '8d837989-4a21-4943-bb4c-4d4abb9a7a55',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_class_helper' => [
				'name' => 'add_class_helper',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADD_CLASS_HELPER_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Helper',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f9b94931-2255-449d-aeab-60bfaa69c449',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_postflight_update' => [
				'name' => 'php_postflight_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_PHP_POSTFLIGHT_UPDATE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'ade283c3-937e-4502-a040-c69d6ddb3c50',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_class_helper_header' => [
				'name' => 'add_class_helper_header',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADD_CLASS_HELPER_HEADER_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Helper',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'd2d81879-9190-45fd-b2a1-9ea6c0b0c12e',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_method_uninstall' => [
				'name' => 'php_method_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_PHP_METHOD_UNINSTALL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e76cf555-112a-4021-9a1e-b4ee52d7be8a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'class_helper_header' => [
				'name' => 'class_helper_header',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_CLASS_HELPER_HEADER_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'Helper',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '1be0f2e4-0270-4ef4-8449-369b9dabcf48',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'sql' => [
				'name' => 'sql',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_SQL_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '5eb879c0-f6e3-45b6-b220-cb5131422a83',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'class_helper_code' => [
				'name' => 'class_helper_code',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_CLASS_HELPER_CODE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'Helper',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '74126149-0257-4844-956d-8f155181ba98',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'sql_uninstall' => [
				'name' => 'sql_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_SQL_UNINSTALL_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'b877b188-693c-43a3-9121-a37a3943d7c1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'fields' => [
				'name' => 'fields',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_FIELDS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'json',
				'tab_name' => 'Forms & Fields',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '22310091-8a1e-4926-846b-caedc3e75b6c',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'module' => [
						'name' => 'module',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'file' => [
						'name' => 'file',
						'type' => 'text',
						'link' => NULL,
					],
					'fields_name' => [
						'name' => 'fields_name',
						'type' => 'text',
						'link' => NULL,
					],
					'fieldset' => [
						'name' => 'fieldset',
						'type' => 'text',
						'link' => NULL,
					],
					'label' => [
						'name' => 'label',
						'type' => 'text',
						'link' => NULL,
					],
					'fields_rules_paths' => [
						'name' => 'fields_rules_paths',
						'type' => 'list',
						'link' => NULL,
					],
					'addrulepath' => [
						'name' => 'addrulepath',
						'type' => 'subform',
						'link' => NULL,
						'fields' => [
							'path' => [
								'name' => 'path',
								'type' => 'text',
								'link' => NULL,
							],
						],
					],
					'addfieldpath' => [
						'name' => 'addfieldpath',
						'type' => 'subform',
						'link' => NULL,
						'fields' => [
							'path' => [
								'name' => 'path',
								'type' => 'text',
								'link' => NULL,
							],
						],
					],
					'fields' => [
						'name' => 'fields',
						'type' => 'subform',
						'link' => NULL,
						'fields' => [
							'field' => [
								'name' => 'field',
								'type' => 'fields',
								'link' => [
									'type' => 1,
									'table' => '#__componentbuilder_field',
									'component' => 'com_componentbuilder',
									'entity' => 'field',
									'value' => 'name',
									'key' => 'guid',
								],
							],
							'custom_value' => [
								'name' => 'custom_value',
								'type' => 'textarea',
								'link' => NULL,
							],
						],
					],
				],
			],
			'readme' => [
				'name' => 'readme',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_README_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'Readme',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '87a4b892-c4cb-4032-8511-e18ba8c8c202',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_script_construct' => [
				'name' => 'add_php_script_construct',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADD_PHP_SCRIPT_CONSTRUCT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f1deeb63-5df2-45ff-b281-a41d9b2cd122',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'update_server_url' => [
				'name' => 'update_server_url',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_UPDATE_SERVER_URL_LABEL',
				'type' => 'url',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'e6c263b1-da39-4a73-8585-637dc3a5b8eb',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_script_construct' => [
				'name' => 'php_script_construct',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_PHP_SCRIPT_CONSTRUCT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '8835dfef-4d87-4104-b93c-47c5e02e4acc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_preflight_install' => [
				'name' => 'add_php_preflight_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADD_PHP_PREFLIGHT_INSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'd5d03647-f29e-44a4-9e63-5feb9f4adbae',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_preflight_install' => [
				'name' => 'php_preflight_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_PHP_PREFLIGHT_INSTALL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '3ca01651-a5c3-46c3-b7a8-bfb0d9f6e105',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_sales_server' => [
				'name' => 'add_sales_server',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADD_SALES_SERVER_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '378a9839-97ce-4bdb-8961-a9164738a3bf',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_preflight_update' => [
				'name' => 'add_php_preflight_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_ADD_PHP_PREFLIGHT_UPDATE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '365d08ee-fe6e-47c4-b567-a001212f2cb6',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_NAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_modules',
				'store' => NULL,
				'tab_name' => 'HTML',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'joomla_plugin' => [
			'system_name' => [
				'name' => 'system_name',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_SYSTEM_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'acfe906b-6e61-4f94-ae66-359e4bc3e4cc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'class_extends' => [
				'name' => 'class_extends',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_CLASS_EXTENDS_LABEL',
				'type' => 'classextends',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'f73b86df-6954-4689-bb74-9af8c9fc0c79',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_class_extends',
					'component' => 'com_componentbuilder',
					'entity' => 'class_extends',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'joomla_plugin_group' => [
				'name' => 'joomla_plugin_group',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_JOOMLA_PLUGIN_GROUP_LABEL',
				'type' => 'joomlaplugingroups',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'd23a041c-6897-4cef-8941-a26a2114a865',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_plugin_group',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_plugin_group',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'add_sql' => [
				'name' => 'add_sql',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_ADD_SQL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '7c652c71-e00e-45ff-8acb-e1bc0344e4c0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_method_uninstall' => [
				'name' => 'add_php_method_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_ADD_PHP_METHOD_UNINSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '70a2d820-64af-4f51-8b1a-522aed9572bf',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_postflight_update' => [
				'name' => 'add_php_postflight_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_ADD_PHP_POSTFLIGHT_UPDATE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '22a668bc-d169-4346-9929-3819673c67ac',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_postflight_install' => [
				'name' => 'add_php_postflight_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_ADD_PHP_POSTFLIGHT_INSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '3c1db8b4-65f2-4f6f-8391-5883f564eabe',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'sales_server' => [
				'name' => 'sales_server',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_SALES_SERVER_LABEL',
				'type' => 'servers',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'GUID' => '4553e8c2-3446-4e11-a967-0504509087a4',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_server',
					'component' => 'com_componentbuilder',
					'entity' => 'server',
					'value' => 'name',
					'key' => 'id',
				],
			],
			'add_update_server' => [
				'name' => 'add_update_server',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_ADD_UPDATE_SERVER_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'db2835aa-b0c3-4c9a-abe6-5b09568fffda',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'method_selection' => [
				'name' => 'method_selection',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_METHOD_SELECTION_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'json',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'e260d101-177f-4722-9b95-417f650f578e',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'method' => [
						'name' => 'method',
						'type' => 'pluginsclassmethods',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_class_method',
							'component' => 'com_componentbuilder',
							'entity' => 'class_method',
							'value' => 'name',
							'key' => 'guid',
						],
					],
				],
			],
			'property_selection' => [
				'name' => 'property_selection',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_PROPERTY_SELECTION_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'json',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'cc01a6b1-7f18-4452-bfa6-4d64fc595a49',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'property' => [
						'name' => 'property',
						'type' => 'pluginsclassproperties',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_class_property',
							'component' => 'com_componentbuilder',
							'entity' => 'class_property',
							'value' => 'name',
							'key' => 'guid',
						],
					],
				],
			],
			'add_head' => [
				'name' => 'add_head',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_ADD_HEAD_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '65341477-33f1-4d58-9b3c-c371168809da',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_sql_uninstall' => [
				'name' => 'add_sql_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_ADD_SQL_UNINSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f22fd59e-4d06-4578-b0bb-a624b8efd145',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'addreadme' => [
				'name' => 'addreadme',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_ADDREADME_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Readme',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '03de1a61-d639-4d48-8ce6-fcf16c8f6270',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'head' => [
				'name' => 'head',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_HEAD_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'base64',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'e4a3ccf7-e259-4b72-ad4c-216b15d6d4f7',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'update_server_target' => [
				'name' => 'update_server_target',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_UPDATE_SERVER_TARGET_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'b9f98c3d-a93a-4b19-b89e-4c62781f9b99',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'main_class_code' => [
				'name' => 'main_class_code',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_MAIN_CLASS_CODE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'base64',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '145966d2-32bf-42a7-8a7b-cc44aa93490d',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'update_server' => [
				'name' => 'update_server',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_UPDATE_SERVER_LABEL',
				'type' => 'servers',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'GUID' => '33fb33c3-1ff2-4d4c-8fe8-6aaa93c1646a',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_server',
					'component' => 'com_componentbuilder',
					'entity' => 'server',
					'value' => 'name',
					'key' => 'id',
				],
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_DESCRIPTION_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '749a9917-90c3-49c4-9e72-aa33b0683a87',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_postflight_install' => [
				'name' => 'php_postflight_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_PHP_POSTFLIGHT_INSTALL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '8d837989-4a21-4943-bb4c-4d4abb9a7a55',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'plugin_version' => [
				'name' => 'plugin_version',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_PLUGIN_VERSION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => '1133b764-a578-4159-abfb-bf3452801974',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_postflight_update' => [
				'name' => 'php_postflight_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_PHP_POSTFLIGHT_UPDATE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'ade283c3-937e-4502-a040-c69d6ddb3c50',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'fields' => [
				'name' => 'fields',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_FIELDS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'json',
				'tab_name' => 'Forms & Fields',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'da319aff-40aa-4cfc-8f0a-2f4184018309',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'plugin' => [
						'name' => 'plugin',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'file' => [
						'name' => 'file',
						'type' => 'text',
						'link' => NULL,
					],
					'fields_name' => [
						'name' => 'fields_name',
						'type' => 'text',
						'link' => NULL,
					],
					'fieldset' => [
						'name' => 'fieldset',
						'type' => 'text',
						'link' => NULL,
					],
					'label' => [
						'name' => 'label',
						'type' => 'text',
						'link' => NULL,
					],
					'fields_rules_paths' => [
						'name' => 'fields_rules_paths',
						'type' => 'list',
						'link' => NULL,
					],
					'addrulepath' => [
						'name' => 'addrulepath',
						'type' => 'subform',
						'link' => NULL,
						'fields' => [
							'path' => [
								'name' => 'path',
								'type' => 'text',
								'link' => NULL,
							],
						],
					],
					'addfieldpath' => [
						'name' => 'addfieldpath',
						'type' => 'subform',
						'link' => NULL,
						'fields' => [
							'path' => [
								'name' => 'path',
								'type' => 'text',
								'link' => NULL,
							],
						],
					],
					'fields' => [
						'name' => 'fields',
						'type' => 'subform',
						'link' => NULL,
						'fields' => [
							'field' => [
								'name' => 'field',
								'type' => 'fields',
								'link' => [
									'type' => 1,
									'table' => '#__componentbuilder_field',
									'component' => 'com_componentbuilder',
									'entity' => 'field',
									'value' => 'name',
									'key' => 'guid',
								],
							],
							'custom_value' => [
								'name' => 'custom_value',
								'type' => 'textarea',
								'link' => NULL,
							],
						],
					],
				],
			],
			'php_method_uninstall' => [
				'name' => 'php_method_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_PHP_METHOD_UNINSTALL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e76cf555-112a-4021-9a1e-b4ee52d7be8a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_script_construct' => [
				'name' => 'add_php_script_construct',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_ADD_PHP_SCRIPT_CONSTRUCT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f1deeb63-5df2-45ff-b281-a41d9b2cd122',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'sql' => [
				'name' => 'sql',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_SQL_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'base64',
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '5eb879c0-f6e3-45b6-b220-cb5131422a83',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_script_construct' => [
				'name' => 'php_script_construct',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_PHP_SCRIPT_CONSTRUCT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '8835dfef-4d87-4104-b93c-47c5e02e4acc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'sql_uninstall' => [
				'name' => 'sql_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_SQL_UNINSTALL_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'base64',
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'b877b188-693c-43a3-9121-a37a3943d7c1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_preflight_install' => [
				'name' => 'add_php_preflight_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_ADD_PHP_PREFLIGHT_INSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'd5d03647-f29e-44a4-9e63-5feb9f4adbae',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'readme' => [
				'name' => 'readme',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_README_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'base64',
				'tab_name' => 'Readme',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '87a4b892-c4cb-4032-8511-e18ba8c8c202',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_preflight_install' => [
				'name' => 'php_preflight_install',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_PHP_PREFLIGHT_INSTALL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '3ca01651-a5c3-46c3-b7a8-bfb0d9f6e105',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'update_server_url' => [
				'name' => 'update_server_url',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_UPDATE_SERVER_URL_LABEL',
				'type' => 'url',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'e6c263b1-da39-4a73-8585-637dc3a5b8eb',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_preflight_update' => [
				'name' => 'add_php_preflight_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_ADD_PHP_PREFLIGHT_UPDATE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '365d08ee-fe6e-47c4-b567-a001212f2cb6',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_preflight_update' => [
				'name' => 'php_preflight_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_PHP_PREFLIGHT_UPDATE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '9f9b030b-31ba-4ae9-88da-76efd8a140dc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_preflight_uninstall' => [
				'name' => 'add_php_preflight_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_ADD_PHP_PREFLIGHT_UNINSTALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '3bb6d77d-05bf-4447-a6bb-4af197ec9290',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_sales_server' => [
				'name' => 'add_sales_server',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_ADD_SALES_SERVER_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Dynamic Integration',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '378a9839-97ce-4bdb-8961-a9164738a3bf',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_preflight_uninstall' => [
				'name' => 'php_preflight_uninstall',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_PHP_PREFLIGHT_UNINSTALL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => 'base64',
				'tab_name' => 'Script File',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '628147c3-5f93-4a49-a1bc-ac6685e06b64',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_NAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_plugins',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'joomla_power' => [
			'system_name' => [
				'name' => 'system_name',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_POWER_SYSTEM_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'joomla_powers',
				'store' => NULL,
				'tab_name' => 'Joomla Power',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'acfe906b-6e61-4f94-ae66-359e4bc3e4cc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_POWER_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_powers',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_POWER_DESCRIPTION_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'joomla_powers',
				'store' => NULL,
				'tab_name' => 'Joomla Power',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '749a9917-90c3-49c4-9e72-aa33b0683a87',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'settings' => [
				'name' => 'settings',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_POWER_SETTINGS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_powers',
				'store' => 'json',
				'tab_name' => 'Joomla Power',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'a1033166-8112-45f9-8af8-c15a53d30c04',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'namespace' => [
						'name' => 'namespace',
						'type' => 'text',
						'link' => NULL,
					],
					'joomla_version' => [
						'name' => 'joomla_version',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'power' => [
			'system_name' => [
				'name' => 'system_name',
				'label' => 'COM_COMPONENTBUILDER_POWER_SYSTEM_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'acfe906b-6e61-4f94-ae66-359e4bc3e4cc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'namespace' => [
				'name' => 'namespace',
				'label' => 'COM_COMPONENTBUILDER_POWER_NAMESPACE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'dde8525b-6cce-4922-a7e0-9dcad5622b4e',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'type' => [
				'name' => 'type',
				'label' => 'COM_COMPONENTBUILDER_POWER_TYPE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'GUID' => '416df962-4468-4665-af1d-da81c9256464',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'power_version' => [
				'name' => 'power_version',
				'label' => 'COM_COMPONENTBUILDER_POWER_POWER_VERSION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => '26e695f7-e7fe-47f4-a692-002dcd2df434',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'load_selection' => [
				'name' => 'load_selection',
				'label' => 'COM_COMPONENTBUILDER_POWER_LOAD_SELECTION_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'powers',
				'store' => 'json',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '6ce1e6cf-264c-4765-aa86-8bcc5f755181',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'load' => [
						'name' => 'load',
						'type' => 'namespaces',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_power',
							'component' => 'com_componentbuilder',
							'entity' => 'power',
							'value' => 'name',
							'key' => 'guid',
						],
					],
				],
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_POWER_DESCRIPTION_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '749a9917-90c3-49c4-9e72-aa33b0683a87',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'composer' => [
				'name' => 'composer',
				'label' => 'COM_COMPONENTBUILDER_POWER_COMPOSER_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'powers',
				'store' => 'json',
				'tab_name' => 'Composer',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'a015aceb-5024-480c-88ea-fd9e4ad2d9f1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'access_point' => [
						'name' => 'access_point',
						'type' => 'text',
						'link' => NULL,
					],
					'namespace' => [
						'name' => 'namespace',
						'type' => 'subform',
						'link' => NULL,
						'fields' => [
							'use' => [
								'name' => 'use',
								'type' => 'text',
								'link' => NULL,
							],
						],
					],
				],
			],
			'licensing_template' => [
				'name' => 'licensing_template',
				'label' => 'COM_COMPONENTBUILDER_POWER_LICENSING_TEMPLATE_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'powers',
				'store' => 'base64',
				'tab_name' => 'Licensing',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'aa4804ce-9099-4006-b089-d05990461668',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'approved' => [
				'name' => 'approved',
				'label' => 'COM_COMPONENTBUILDER_POWER_APPROVED_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'Super Power',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'a0e9a1e1-d32b-4bb7-b3e8-e146b5427405',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'extendsinterfaces_custom' => [
				'name' => 'extendsinterfaces_custom',
				'label' => 'COM_COMPONENTBUILDER_POWER_EXTENDSINTERFACES_CUSTOM_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '45a7e8c9-6e44-4e0c-b0c5-3055cdb81a12',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_head' => [
				'name' => 'add_head',
				'label' => 'COM_COMPONENTBUILDER_POWER_ADD_HEAD_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '65341477-33f1-4d58-9b3c-c371168809da',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'extends' => [
				'name' => 'extends',
				'label' => 'COM_COMPONENTBUILDER_POWER_EXTENDS_LABEL',
				'type' => 'classpowers',
				'title' => false,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'd42f8ec2-710c-41a9-98b6-b74b19213cdb',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_power',
					'component' => 'com_componentbuilder',
					'entity' => 'power',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'extends_custom' => [
				'name' => 'extends_custom',
				'label' => 'COM_COMPONENTBUILDER_POWER_EXTENDS_CUSTOM_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'GUID' => '1a0ff40c-90bc-472f-a465-8afefb595a6b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'implements_custom' => [
				'name' => 'implements_custom',
				'label' => 'COM_COMPONENTBUILDER_POWER_IMPLEMENTS_CUSTOM_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'VARCHAR(1024)',
					'default' => '',
					'GUID' => '68dcb3dc-15b2-4357-8c87-f2fb316e9ae0',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'implements' => [
				'name' => 'implements',
				'label' => 'COM_COMPONENTBUILDER_POWER_IMPLEMENTS_LABEL',
				'type' => 'interfacepowers',
				'title' => false,
				'list' => 'powers',
				'store' => 'json',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '82b08f94-4ed2-40cf-a311-e323e69d47b1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_power',
					'component' => 'com_componentbuilder',
					'entity' => 'power',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'property_selection' => [
				'name' => 'property_selection',
				'label' => 'COM_COMPONENTBUILDER_POWER_PROPERTY_SELECTION_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'powers',
				'store' => 'json',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'bdd69212-bb69-40b1-8a16-6b665ae94a36',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'property' => [
						'name' => 'property',
						'type' => 'powersclassproperties',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_class_property',
							'component' => 'com_componentbuilder',
							'entity' => 'class_property',
							'value' => 'name',
							'key' => 'guid',
						],
					],
				],
			],
			'extendsinterfaces' => [
				'name' => 'extendsinterfaces',
				'label' => 'COM_COMPONENTBUILDER_POWER_EXTENDSINTERFACES_LABEL',
				'type' => 'interfacepowers',
				'title' => false,
				'list' => 'powers',
				'store' => 'json',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '4e23487e-5052-4f04-9352-15c4b75031d8',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_power',
					'component' => 'com_componentbuilder',
					'entity' => 'power',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'method_selection' => [
				'name' => 'method_selection',
				'label' => 'COM_COMPONENTBUILDER_POWER_METHOD_SELECTION_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'powers',
				'store' => 'json',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '645ed24e-5159-40a2-bd50-4aa26e625848',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'method' => [
						'name' => 'method',
						'type' => 'powersclassmethods',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_class_method',
							'component' => 'com_componentbuilder',
							'entity' => 'class_method',
							'value' => 'name',
							'key' => 'guid',
						],
					],
				],
			],
			'approved_paths' => [
				'name' => 'approved_paths',
				'label' => 'COM_COMPONENTBUILDER_POWER_APPROVED_PATHS_LABEL',
				'type' => 'superpowerpaths',
				'title' => false,
				'list' => 'powers',
				'store' => 'json',
				'tab_name' => 'Super Power',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '7ee9f69e-1b64-45c6-90db-d0558f6e40e7',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 2,
					'table' => NULL,
					'component' => NULL,
					'entity' => NULL,
					'value' => NULL,
					'key' => NULL,
				],
			],
			'head' => [
				'name' => 'head',
				'label' => 'COM_COMPONENTBUILDER_POWER_HEAD_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'powers',
				'store' => 'base64',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'e4a3ccf7-e259-4b72-ad4c-216b15d6d4f7',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'use_selection' => [
				'name' => 'use_selection',
				'label' => 'COM_COMPONENTBUILDER_POWER_USE_SELECTION_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'powers',
				'store' => 'json',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'f43c4b83-952d-4100-8b9c-688427df59dd',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'use' => [
						'name' => 'use',
						'type' => 'namespaces',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_power',
							'component' => 'com_componentbuilder',
							'entity' => 'power',
							'value' => 'name',
							'key' => 'guid',
						],
					],
					'as' => [
						'name' => 'as',
						'type' => 'text',
						'link' => NULL,
					],
				],
			],
			'add_licensing_template' => [
				'name' => 'add_licensing_template',
				'label' => 'COM_COMPONENTBUILDER_POWER_ADD_LICENSING_TEMPLATE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'Licensing',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '1',
					'GUID' => 'd937bf3c-2b58-4df4-a6a5-b497bb7b8245',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'main_class_code' => [
				'name' => 'main_class_code',
				'label' => 'COM_COMPONENTBUILDER_POWER_MAIN_CLASS_CODE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'powers',
				'store' => 'base64',
				'tab_name' => 'Code',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '145966d2-32bf-42a7-8a7b-cc44aa93490d',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_POWER_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_POWER_NAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'powers',
				'store' => NULL,
				'tab_name' => 'Code',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'admin_view' => [
			'system_name' => [
				'name' => 'system_name',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_SYSTEM_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'acfe906b-6e61-4f94-ae66-359e4bc3e4cc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'name_single' => [
				'name' => 'name_single',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_NAME_SINGLE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => '7568fcc2-bd06-438c-9666-fe3bde32306f',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'short_description' => [
				'name' => 'short_description',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_SHORT_DESCRIPTION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '46503007-43d9-485f-8cd0-8427a6a8bef2',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_allowedit' => [
				'name' => 'php_allowedit',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_ALLOWEDIT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '88173ddb-35c5-49a7-845d-0d4e12119a37',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_postsavehook' => [
				'name' => 'php_postsavehook',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_POSTSAVEHOOK_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'b97daa06-7589-43e1-aaf9-c9506756f7b4',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_before_save' => [
				'name' => 'php_before_save',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_BEFORE_SAVE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '7dd15ec3-1830-4f4c-9497-d33618702263',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_getlistquery' => [
				'name' => 'php_getlistquery',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_GETLISTQUERY_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'b9c064cb-4266-4217-95b7-d2956e6bfefb',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_getitems' => [
				'name' => 'php_getitems',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_GETITEMS_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '0ec2dbcd-8acc-438f-ad28-906fc7b6474e',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'name_list' => [
				'name' => 'name_list',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_NAME_LIST_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => 'c73c3033-f4d9-475a-977b-9e447b88272f',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'icon' => [
				'name' => 'icon',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ICON_LABEL',
				'type' => 'media',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '3b731d0a-3205-4409-a9fd-e74d1db25055',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_after_publish' => [
				'name' => 'php_after_publish',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_AFTER_PUBLISH_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '8dfde218-c13c-4e7e-be62-d651cafc9d43',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_fadein' => [
				'name' => 'add_fadein',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_FADEIN_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'e1bcb366-b7e0-47d7-9a5f-58f8d8d0307d',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_DESCRIPTION_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '749a9917-90c3-49c4-9e72-aa33b0683a87',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'icon_category' => [
				'name' => 'icon_category',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ICON_CATEGORY_LABEL',
				'type' => 'media',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '0d2bdd49-7559-4cc1-a25b-56aacb830760',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'icon_add' => [
				'name' => 'icon_add',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ICON_ADD_LABEL',
				'type' => 'media',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '92ff3e2c-081b-4b84-a10c-e1a9f3245063',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_after_cancel' => [
				'name' => 'php_after_cancel',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_AFTER_CANCEL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '5637623b-c82f-4ce2-be82-e262e9596fa2',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'mysql_table_charset' => [
				'name' => 'mysql_table_charset',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_MYSQL_TABLE_CHARSET_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '1',
					'GUID' => '916b0b6e-b5dc-469e-bc9c-c809471903b4',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_batchmove' => [
				'name' => 'php_batchmove',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_BATCHMOVE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'c56cf55d-fbc6-4246-b3c3-7810d42710b2',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'type' => [
				'name' => 'type',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_TYPE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'GUID' => '5f2ddc22-11f5-49a5-bebf-8313d8006e46',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_after_delete' => [
				'name' => 'php_after_delete',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_AFTER_DELETE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'f23fc722-e0e2-422f-b0f5-ac17da162a63',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'source' => [
				'name' => 'source',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_SOURCE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '21eff77d-da38-4ce1-96c5-e6af26463942',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_getitem' => [
				'name' => 'php_getitem',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_GETITEM_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'a921c373-45bb-4919-a840-5b13511102cb',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'addpermissions' => [
				'name' => 'addpermissions',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADDPERMISSIONS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'json',
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '6e7d306a-9210-41bd-b951-6e897fcff773',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'action' => [
						'name' => 'action',
						'type' => 'list',
						'link' => NULL,
					],
					'implementation' => [
						'name' => 'implementation',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'php_getitems_after_all' => [
				'name' => 'php_getitems_after_all',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_GETITEMS_AFTER_ALL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '887264da-8ea1-4333-8a12-de26f6041251',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_getform' => [
				'name' => 'php_getform',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_GETFORM_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '120e7c82-ff53-44ec-bfa0-a1413d100d18',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'addtabs' => [
				'name' => 'addtabs',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADDTABS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'json',
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '0c06ec4b-448e-4c8a-8869-d424b6ab1f4a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'name' => [
						'name' => 'name',
						'type' => 'text',
						'link' => NULL,
					],
				],
			],
			'php_save' => [
				'name' => 'php_save',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_SAVE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '8ac3beb5-c100-41d6-8d2e-dd3e57f3701a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_allowadd' => [
				'name' => 'php_allowadd',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_ALLOWADD_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '2f0432b4-e601-474e-9edb-4b438517039b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_before_cancel' => [
				'name' => 'php_before_cancel',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_BEFORE_CANCEL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '0523633b-2f83-4205-ad7a-f4a77dff3dd0',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'addlinked_views' => [
				'name' => 'addlinked_views',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADDLINKED_VIEWS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'json',
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '2e7df4e5-488d-4ea5-9ae3-42c7bf5f926e',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'adminview' => [
						'name' => 'adminview',
						'type' => 'adminviews',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_admin_view',
							'component' => 'com_componentbuilder',
							'entity' => 'admin_view',
							'value' => 'system_name',
							'key' => 'guid',
						],
					],
					'tab' => [
						'name' => 'tab',
						'type' => 'integer',
						'link' => NULL,
					],
					'key' => [
						'name' => 'key',
						'type' => 'text',
						'link' => NULL,
					],
					'parentkey' => [
						'name' => 'parentkey',
						'type' => 'text',
						'link' => NULL,
					],
					'addnew' => [
						'name' => 'addnew',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'php_batchcopy' => [
				'name' => 'php_batchcopy',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_BATCHCOPY_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '77067bb3-3cfa-4042-b3ac-8bdda9d8b492',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_before_publish' => [
				'name' => 'php_before_publish',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_BEFORE_PUBLISH_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e4b56d60-7171-43c9-89ad-03f04a2cdba1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'alias_builder_type' => [
				'name' => 'alias_builder_type',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ALIAS_BUILDER_TYPE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'Fields',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'a5043582-b29c-4ede-a712-16de9346abed',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_before_delete' => [
				'name' => 'php_before_delete',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_BEFORE_DELETE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '8e8e12f8-8d8c-43f1-bbbd-8ef19b4d326e',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_document' => [
				'name' => 'php_document',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_DOCUMENT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e4d3c22b-1297-450b-8108-c389f834e354',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'mysql_table_row_format' => [
				'name' => 'mysql_table_row_format',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_MYSQL_TABLE_ROW_FORMAT_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '0',
					'GUID' => '6336be5a-77e0-46e2-8d1f-9278a4b10080',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'alias_builder' => [
				'name' => 'alias_builder',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ALIAS_BUILDER_LABEL',
				'type' => 'aliasbuilder',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'json',
				'tab_name' => 'Fields',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '96e3d0e6-e8b5-488f-8758-7cec18bd5858',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_field',
					'component' => 'com_componentbuilder',
					'entity' => 'field',
					'value' => 'name',
					'key' => 'id',
				],
			],
			'sql' => [
				'name' => 'sql',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_SQL_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '5eb879c0-f6e3-45b6-b220-cb5131422a83',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_getitem' => [
				'name' => 'add_php_getitem',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_GETITEM_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '95eb062e-f221-43b4-b4d4-c32e9d70e8a7',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_category_submenu' => [
				'name' => 'add_category_submenu',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_CATEGORY_SUBMENU_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'Fields',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '1',
					'GUID' => '7e6e7fa8-d17b-47ae-9a28-b02ea4936b81',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_getitems' => [
				'name' => 'add_php_getitems',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_GETITEMS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '1eecc354-f454-48dd-ab12-4cf85b88f97b',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_getitems_after_all' => [
				'name' => 'add_php_getitems_after_all',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_GETITEMS_AFTER_ALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '4b956c45-bbcd-4d01-82ce-a85270f44c10',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_getlistquery' => [
				'name' => 'add_php_getlistquery',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_GETLISTQUERY_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f2581140-5d9c-4c29-b511-e12a826d15da',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_css_view' => [
				'name' => 'add_css_view',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_CSS_VIEW_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'CSS',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '6746d3ba-b120-4918-8bba-2a14260e99c8',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_getform' => [
				'name' => 'add_php_getform',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_GETFORM_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '562bc505-cc89-4a55-956a-b53e1a87ef46',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'css_view' => [
				'name' => 'css_view',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_CSS_VIEW_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'CSS',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '1ad12721-9ee9-4623-98d2-071ba013a373',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_before_save' => [
				'name' => 'add_php_before_save',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_BEFORE_SAVE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'cc7954c1-d9f7-4a97-8b71-23f6f694bd6d',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_css_views' => [
				'name' => 'add_css_views',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_CSS_VIEWS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'CSS',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'a2098911-926a-42b6-8498-d8a453c34d93',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_save' => [
				'name' => 'add_php_save',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_SAVE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '5799fcb7-8de8-41bf-9a2a-dcf84cae3949',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'css_views' => [
				'name' => 'css_views',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_CSS_VIEWS_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'CSS',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '63d5585f-c9bd-485d-9c1d-4a29a4fbf4b2',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_postsavehook' => [
				'name' => 'add_php_postsavehook',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_POSTSAVEHOOK_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '45559d2a-7537-4719-8caa-aa99df171000',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_javascript_view_file' => [
				'name' => 'add_javascript_view_file',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_JAVASCRIPT_VIEW_FILE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'JavaScript',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '42991cef-d988-4ad9-b3d2-f29d9809f0b3',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_allowadd' => [
				'name' => 'add_php_allowadd',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_ALLOWADD_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '4550b372-974e-432c-8221-69b6fb65fdfd',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'javascript_view_file' => [
				'name' => 'javascript_view_file',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_JAVASCRIPT_VIEW_FILE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'JavaScript',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '5f7479f2-b499-4094-8294-d580e51244a1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_allowedit' => [
				'name' => 'add_php_allowedit',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_ALLOWEDIT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '30744253-e2cd-44c3-ac86-0de3741b9ce0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_javascript_view_footer' => [
				'name' => 'add_javascript_view_footer',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_JAVASCRIPT_VIEW_FOOTER_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'JavaScript',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '9405da44-900c-4f7c-823d-734a68f5d6f6',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_before_cancel' => [
				'name' => 'add_php_before_cancel',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_BEFORE_CANCEL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '5b8bfcb4-a2b0-4ee9-931e-474ae9742e76',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'javascript_view_footer' => [
				'name' => 'javascript_view_footer',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_JAVASCRIPT_VIEW_FOOTER_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'JavaScript',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'a99c1f40-4b41-4167-ad29-85133375aa7a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_after_cancel' => [
				'name' => 'add_php_after_cancel',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_AFTER_CANCEL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '1e5269aa-e342-4b63-830e-f0a7acb2dab4',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_javascript_views_file' => [
				'name' => 'add_javascript_views_file',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_JAVASCRIPT_VIEWS_FILE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'JavaScript',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'e19c4fa4-804f-488b-90d7-1ea827bba52e',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_batchcopy' => [
				'name' => 'add_php_batchcopy',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_BATCHCOPY_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '0c7535bd-dbfa-4155-bc06-9ef34bf0f29a',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'javascript_views_file' => [
				'name' => 'javascript_views_file',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_JAVASCRIPT_VIEWS_FILE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'JavaScript',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'ec2db8b1-d779-4bf2-b4cd-374a894fdc83',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_batchmove' => [
				'name' => 'add_php_batchmove',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_BATCHMOVE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '90c08eb4-3097-4f59-a291-5959bbb174ad',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_javascript_views_footer' => [
				'name' => 'add_javascript_views_footer',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_JAVASCRIPT_VIEWS_FOOTER_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'JavaScript',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '70995b9c-d0a9-4a57-b59b-54d39c379e66',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_before_publish' => [
				'name' => 'add_php_before_publish',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_BEFORE_PUBLISH_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '3b02a65c-1a0a-4e04-8306-34cb484c45fe',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'javascript_views_footer' => [
				'name' => 'javascript_views_footer',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_JAVASCRIPT_VIEWS_FOOTER_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'JavaScript',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '34bf9926-34a4-49a3-a29e-b602b1e5f9b2',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_after_publish' => [
				'name' => 'add_php_after_publish',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_AFTER_PUBLISH_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '3eb7643a-7205-4683-9b8b-80c2d65334eb',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_custom_button' => [
				'name' => 'add_custom_button',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_CUSTOM_BUTTON_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'INT(1)',
					'default' => '0',
					'GUID' => 'ab222d85-e408-44de-a513-d6aeda48b286',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_before_delete' => [
				'name' => 'add_php_before_delete',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_BEFORE_DELETE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '79a25c26-9ea4-481c-b833-6699dd6e34d0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'custom_button' => [
				'name' => 'custom_button',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_CUSTOM_BUTTON_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'json',
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'f962e778-623e-40a8-b7db-88df71fdac51',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'icomoon' => [
						'name' => 'icomoon',
						'type' => 'list',
						'link' => NULL,
					],
					'name' => [
						'name' => 'name',
						'type' => 'text',
						'link' => NULL,
					],
					'method' => [
						'name' => 'method',
						'type' => 'text',
						'link' => NULL,
					],
					'target' => [
						'name' => 'target',
						'type' => 'list',
						'link' => NULL,
					],
					'type' => [
						'name' => 'type',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'add_php_after_delete' => [
				'name' => 'add_php_after_delete',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_AFTER_DELETE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '8272a897-9044-4e62-964f-89cd90e99bbb',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_controller' => [
				'name' => 'php_controller',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_CONTROLLER_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e268c7be-dc34-40d9-bf2c-5f1897a09f7c',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_document' => [
				'name' => 'add_php_document',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_DOCUMENT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '46fa9c1e-5aa3-41ff-8f02-e9a80252ad9f',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_model' => [
				'name' => 'php_model',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_MODEL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '5e831983-665b-43f2-9786-5e7c9cbb9e96',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'mysql_table_engine' => [
				'name' => 'mysql_table_engine',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_MYSQL_TABLE_ENGINE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '1',
					'GUID' => '025cfab0-7736-4036-9eec-8f70e53f681b',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_controller_list' => [
				'name' => 'php_controller_list',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_CONTROLLER_LIST_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'fe8a8811-1bcd-448a-a111-25e17ac47687',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'mysql_table_collate' => [
				'name' => 'mysql_table_collate',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_MYSQL_TABLE_COLLATE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '1',
					'GUID' => '29e8af00-5d33-4ddc-8643-9f331da7684e',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_model_list' => [
				'name' => 'php_model_list',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_MODEL_LIST_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '46e5c9d6-6304-4e9f-a582-242308667e2f',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_sql' => [
				'name' => 'add_sql',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_SQL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '7c652c71-e00e-45ff-8acb-e1bc0344e4c0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_ajax' => [
				'name' => 'add_php_ajax',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADD_PHP_AJAX_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '625b65cb-7569-437a-a6dd-b3290c662e18',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'addtables' => [
				'name' => 'addtables',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_ADDTABLES_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'json',
				'tab_name' => 'MySQL',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '33a9df05-3e39-4cc8-af2b-234653aadb7a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'table' => [
						'name' => 'table',
						'type' => 'dbtables',
						'link' => [
							'type' => 2,
							'table' => '',
							'component' => '',
							'entity' => '',
							'value' => '',
							'key' => '',
						],
					],
					'sourcemap' => [
						'name' => 'sourcemap',
						'type' => 'textarea',
						'link' => NULL,
					],
				],
			],
			'php_ajaxmethod' => [
				'name' => 'php_ajaxmethod',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_PHP_AJAXMETHOD_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'c731b5f5-5bd3-4ad3-ac3e-9e0281104121',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'admin_views',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'ajax_input' => [
				'name' => 'ajax_input',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_VIEW_AJAX_INPUT_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'admin_views',
				'store' => 'json',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'e407f6f1-e4e9-4f23-a8c7-8ad75e14a820',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'value_name' => [
						'name' => 'value_name',
						'type' => 'text',
						'link' => NULL,
					],
					'task_name' => [
						'name' => 'task_name',
						'type' => 'text',
						'link' => NULL,
					],
					'input_default' => [
						'name' => 'input_default',
						'type' => 'text',
						'link' => NULL,
					],
					'input_filter' => [
						'name' => 'input_filter',
						'type' => 'list',
						'link' => NULL,
					],
					'method_name' => [
						'name' => 'method_name',
						'type' => 'text',
						'link' => NULL,
					],
					'allow_zero' => [
						'name' => 'allow_zero',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'user_check' => [
						'name' => 'user_check',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'custom_admin_view' => [
			'system_name' => [
				'name' => 'system_name',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_SYSTEM_NAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'acfe906b-6e61-4f94-ae66-359e4bc3e4cc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_DESCRIPTION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'a5ef0756-5ecd-49ae-a36c-058b90758988',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'main_get' => [
				'name' => 'main_get',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_MAIN_GET_LABEL',
				'type' => 'maingets',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'd20027fe-4e74-4261-9bf2-7e573a3e883b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_dynamic_get',
					'component' => 'com_componentbuilder',
					'entity' => 'dynamic_get',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'add_php_jview_display' => [
				'name' => 'add_php_jview_display',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_ADD_PHP_JVIEW_DISPLAY_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '32564da8-5599-4577-bd93-fd425179612f',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'css_document' => [
				'name' => 'css_document',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_CSS_DOCUMENT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'base64',
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'fd03122f-8c74-468a-8f8c-704e927c3b65',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'css' => [
				'name' => 'css',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_CSS_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'base64',
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '7d733ad8-a5bf-4e2f-859d-be1251b7d0f1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'js_document' => [
				'name' => 'js_document',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_JS_DOCUMENT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'base64',
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'ccb3cb86-8403-4494-a284-ce1fb782f0af',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'javascript_file' => [
				'name' => 'javascript_file',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_JAVASCRIPT_FILE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'base64',
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '25c87995-f58f-4f6a-8f6b-2e462545bdd6',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'codename' => [
				'name' => 'codename',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_CODENAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '83871f4f-e406-416a-845e-a8257a4a34d3',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'default' => [
				'name' => 'default',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_DEFAULT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '6e214a72-6c96-40c0-abf3-5f09116d3018',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'snippet' => [
				'name' => 'snippet',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_SNIPPET_LABEL',
				'type' => 'snippets',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'fc4bc2f3-1a12-4484-88dc-273199fe3b63',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_snippet',
					'component' => 'com_componentbuilder',
					'entity' => 'snippet',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'icon' => [
				'name' => 'icon',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_ICON_LABEL',
				'type' => 'media',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '3b731d0a-3205-4409-a9fd-e74d1db25055',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_jview' => [
				'name' => 'add_php_jview',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_ADD_PHP_JVIEW_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '6ce6e81e-5cdf-4ea2-83b5-acde779f4297',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'context' => [
				'name' => 'context',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_CONTEXT_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'c7b20fcd-0849-49cc-84c2-0dbb73a19fda',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_js_document' => [
				'name' => 'add_js_document',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_ADD_JS_DOCUMENT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f76ffc06-f4c9-417e-8584-bc2f65184d18',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'custom_get' => [
				'name' => 'custom_get',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_CUSTOM_GET_LABEL',
				'type' => 'customgets',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'bed7693b-ccd1-4803-b065-5e20fac13881',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_dynamic_get',
					'component' => 'com_componentbuilder',
					'entity' => 'dynamic_get',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'add_javascript_file' => [
				'name' => 'add_javascript_file',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_ADD_JAVASCRIPT_FILE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '075b5082-3fb1-46f1-a838-da558b3afb54',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_ajaxmethod' => [
				'name' => 'php_ajaxmethod',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_PHP_AJAXMETHOD_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'c731b5f5-5bd3-4ad3-ac3e-9e0281104121',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_css_document' => [
				'name' => 'add_css_document',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_ADD_CSS_DOCUMENT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '4a8003fa-7458-405e-90d3-7728b1af9e86',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_document' => [
				'name' => 'add_php_document',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_ADD_PHP_DOCUMENT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '46fa9c1e-5aa3-41ff-8f02-e9a80252ad9f',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_css' => [
				'name' => 'add_css',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_ADD_CSS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '14c857e3-e684-49a3-8b34-1cf7448ed0c4',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_view' => [
				'name' => 'add_php_view',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_ADD_PHP_VIEW_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '8b09aaf9-546c-4f03-8b1f-3170ba4b5491',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_ajax' => [
				'name' => 'add_php_ajax',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_ADD_PHP_AJAX_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '625b65cb-7569-437a-a6dd-b3290c662e18',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'libraries' => [
				'name' => 'libraries',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_LIBRARIES_LABEL',
				'type' => 'libraries',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'f17742b9-0096-4b03-b981-f0dcd3f7e8c1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_library',
					'component' => 'com_componentbuilder',
					'entity' => 'library',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'ajax_input' => [
				'name' => 'ajax_input',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_AJAX_INPUT_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'json',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'e407f6f1-e4e9-4f23-a8c7-8ad75e14a820',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'value_name' => [
						'name' => 'value_name',
						'type' => 'text',
						'link' => NULL,
					],
					'task_name' => [
						'name' => 'task_name',
						'type' => 'text',
						'link' => NULL,
					],
					'input_default' => [
						'name' => 'input_default',
						'type' => 'text',
						'link' => NULL,
					],
					'input_filter' => [
						'name' => 'input_filter',
						'type' => 'list',
						'link' => NULL,
					],
					'method_name' => [
						'name' => 'method_name',
						'type' => 'text',
						'link' => NULL,
					],
					'allow_zero' => [
						'name' => 'allow_zero',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'user_check' => [
						'name' => 'user_check',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'dynamic_get' => [
				'name' => 'dynamic_get',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_DYNAMIC_GET_LABEL',
				'type' => 'dynamicgets',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'c37aa662-719c-4ed5-bbf2-e36672a3ce33',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_dynamic_get',
					'component' => 'com_componentbuilder',
					'entity' => 'dynamic_get',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'php_document' => [
				'name' => 'php_document',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_PHP_DOCUMENT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e4d3c22b-1297-450b-8108-c389f834e354',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_view' => [
				'name' => 'php_view',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_PHP_VIEW_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e27df414-cf8a-45c3-bc38-cc48e6146fef',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_custom_button' => [
				'name' => 'add_custom_button',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_ADD_CUSTOM_BUTTON_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'INT(1)',
					'default' => '0',
					'GUID' => 'ab222d85-e408-44de-a513-d6aeda48b286',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_jview_display' => [
				'name' => 'php_jview_display',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_PHP_JVIEW_DISPLAY_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '96963615-e3bb-4d37-83df-4ad3fca74536',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'custom_button' => [
				'name' => 'custom_button',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_CUSTOM_BUTTON_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'json',
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'f962e778-623e-40a8-b7db-88df71fdac51',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'icomoon' => [
						'name' => 'icomoon',
						'type' => 'list',
						'link' => NULL,
					],
					'name' => [
						'name' => 'name',
						'type' => 'text',
						'link' => NULL,
					],
					'method' => [
						'name' => 'method',
						'type' => 'text',
						'link' => NULL,
					],
					'target' => [
						'name' => 'target',
						'type' => 'list',
						'link' => NULL,
					],
					'type' => [
						'name' => 'type',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'php_jview' => [
				'name' => 'php_jview',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_PHP_JVIEW_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'ffecfa82-c326-4703-9cc3-69d6f313bd32',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_controller' => [
				'name' => 'php_controller',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_PHP_CONTROLLER_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'base64',
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e268c7be-dc34-40d9-bf2c-5f1897a09f7c',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_model' => [
				'name' => 'php_model',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_ADMIN_VIEW_PHP_MODEL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'custom_admin_views',
				'store' => 'base64',
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '5e831983-665b-43f2-9786-5e7c9cbb9e96',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'site_view' => [
			'system_name' => [
				'name' => 'system_name',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_SYSTEM_NAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'acfe906b-6e61-4f94-ae66-359e4bc3e4cc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_DESCRIPTION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'a5ef0756-5ecd-49ae-a36c-058b90758988',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'main_get' => [
				'name' => 'main_get',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_MAIN_GET_LABEL',
				'type' => 'maingets',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'd20027fe-4e74-4261-9bf2-7e573a3e883b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_dynamic_get',
					'component' => 'com_componentbuilder',
					'entity' => 'dynamic_get',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'add_php_jview_display' => [
				'name' => 'add_php_jview_display',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_ADD_PHP_JVIEW_DISPLAY_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '32564da8-5599-4577-bd93-fd425179612f',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_document' => [
				'name' => 'add_php_document',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_ADD_PHP_DOCUMENT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '46fa9c1e-5aa3-41ff-8f02-e9a80252ad9f',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_view' => [
				'name' => 'add_php_view',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_ADD_PHP_VIEW_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '8b09aaf9-546c-4f03-8b1f-3170ba4b5491',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'js_document' => [
				'name' => 'js_document',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_JS_DOCUMENT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'site_views',
				'store' => 'base64',
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'ccb3cb86-8403-4494-a284-ce1fb782f0af',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'codename' => [
				'name' => 'codename',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_CODENAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '83871f4f-e406-416a-845e-a8257a4a34d3',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'javascript_file' => [
				'name' => 'javascript_file',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_JAVASCRIPT_FILE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'site_views',
				'store' => 'base64',
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '25c87995-f58f-4f6a-8f6b-2e462545bdd6',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'context' => [
				'name' => 'context',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_CONTEXT_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'c7b20fcd-0849-49cc-84c2-0dbb73a19fda',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'default' => [
				'name' => 'default',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_DEFAULT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'site_views',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '6e214a72-6c96-40c0-abf3-5f09116d3018',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'snippet' => [
				'name' => 'snippet',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_SNIPPET_LABEL',
				'type' => 'snippets',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'fc4bc2f3-1a12-4484-88dc-273199fe3b63',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_snippet',
					'component' => 'com_componentbuilder',
					'entity' => 'snippet',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'add_php_jview' => [
				'name' => 'add_php_jview',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_ADD_PHP_JVIEW_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '6ce6e81e-5cdf-4ea2-83b5-acde779f4297',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'custom_get' => [
				'name' => 'custom_get',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_CUSTOM_GET_LABEL',
				'type' => 'customgets',
				'title' => false,
				'list' => 'site_views',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'bed7693b-ccd1-4803-b065-5e20fac13881',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_dynamic_get',
					'component' => 'com_componentbuilder',
					'entity' => 'dynamic_get',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'css_document' => [
				'name' => 'css_document',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_CSS_DOCUMENT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'site_views',
				'store' => 'base64',
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'fd03122f-8c74-468a-8f8c-704e927c3b65',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_javascript_file' => [
				'name' => 'add_javascript_file',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_ADD_JAVASCRIPT_FILE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '075b5082-3fb1-46f1-a838-da558b3afb54',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'css' => [
				'name' => 'css',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_CSS_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'site_views',
				'store' => 'base64',
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '7d733ad8-a5bf-4e2f-859d-be1251b7d0f1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_js_document' => [
				'name' => 'add_js_document',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_ADD_JS_DOCUMENT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f76ffc06-f4c9-417e-8584-bc2f65184d18',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_ajaxmethod' => [
				'name' => 'php_ajaxmethod',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_PHP_AJAXMETHOD_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'site_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'c731b5f5-5bd3-4ad3-ac3e-9e0281104121',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_css_document' => [
				'name' => 'add_css_document',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_ADD_CSS_DOCUMENT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '4a8003fa-7458-405e-90d3-7728b1af9e86',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'libraries' => [
				'name' => 'libraries',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_LIBRARIES_LABEL',
				'type' => 'libraries',
				'title' => false,
				'list' => 'site_views',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'f17742b9-0096-4b03-b981-f0dcd3f7e8c1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_library',
					'component' => 'com_componentbuilder',
					'entity' => 'library',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'add_css' => [
				'name' => 'add_css',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_ADD_CSS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'JavaScript & CSS',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '14c857e3-e684-49a3-8b34-1cf7448ed0c4',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'dynamic_get' => [
				'name' => 'dynamic_get',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_DYNAMIC_GET_LABEL',
				'type' => 'dynamicgets',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'c37aa662-719c-4ed5-bbf2-e36672a3ce33',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_dynamic_get',
					'component' => 'com_componentbuilder',
					'entity' => 'dynamic_get',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'add_php_ajax' => [
				'name' => 'add_php_ajax',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_ADD_PHP_AJAX_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '625b65cb-7569-437a-a6dd-b3290c662e18',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'ajax_input' => [
				'name' => 'ajax_input',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_AJAX_INPUT_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'site_views',
				'store' => 'json',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'e407f6f1-e4e9-4f23-a8c7-8ad75e14a820',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'value_name' => [
						'name' => 'value_name',
						'type' => 'text',
						'link' => NULL,
					],
					'task_name' => [
						'name' => 'task_name',
						'type' => 'text',
						'link' => NULL,
					],
					'input_default' => [
						'name' => 'input_default',
						'type' => 'text',
						'link' => NULL,
					],
					'input_filter' => [
						'name' => 'input_filter',
						'type' => 'list',
						'link' => NULL,
					],
					'method_name' => [
						'name' => 'method_name',
						'type' => 'text',
						'link' => NULL,
					],
					'allow_zero' => [
						'name' => 'allow_zero',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'user_check' => [
						'name' => 'user_check',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'add_custom_button' => [
				'name' => 'add_custom_button',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_ADD_CUSTOM_BUTTON_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'INT(1)',
					'default' => '0',
					'GUID' => 'ab222d85-e408-44de-a513-d6aeda48b286',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_document' => [
				'name' => 'php_document',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_PHP_DOCUMENT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'site_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e4d3c22b-1297-450b-8108-c389f834e354',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'button_position' => [
				'name' => 'button_position',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_BUTTON_POSITION_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '1',
					'GUID' => '29a26946-d7c3-4ea0-be64-8e376d2a7f25',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_view' => [
				'name' => 'php_view',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_PHP_VIEW_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'site_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e27df414-cf8a-45c3-bc38-cc48e6146fef',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_jview_display' => [
				'name' => 'php_jview_display',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_PHP_JVIEW_DISPLAY_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'site_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '96963615-e3bb-4d37-83df-4ad3fca74536',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'custom_button' => [
				'name' => 'custom_button',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_CUSTOM_BUTTON_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'site_views',
				'store' => 'json',
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'f962e778-623e-40a8-b7db-88df71fdac51',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'icomoon' => [
						'name' => 'icomoon',
						'type' => 'list',
						'link' => NULL,
					],
					'name' => [
						'name' => 'name',
						'type' => 'text',
						'link' => NULL,
					],
					'method' => [
						'name' => 'method',
						'type' => 'text',
						'link' => NULL,
					],
					'target' => [
						'name' => 'target',
						'type' => 'list',
						'link' => NULL,
					],
					'type' => [
						'name' => 'type',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'php_jview' => [
				'name' => 'php_jview',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_PHP_JVIEW_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'site_views',
				'store' => 'base64',
				'tab_name' => 'PHP',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'ffecfa82-c326-4703-9cc3-69d6f313bd32',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_controller' => [
				'name' => 'php_controller',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_PHP_CONTROLLER_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'site_views',
				'store' => 'base64',
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e268c7be-dc34-40d9-bf2c-5f1897a09f7c',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'site_views',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_model' => [
				'name' => 'php_model',
				'label' => 'COM_COMPONENTBUILDER_SITE_VIEW_PHP_MODEL_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'site_views',
				'store' => 'base64',
				'tab_name' => 'Custom Buttons',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '5e831983-665b-43f2-9786-5e7c9cbb9e96',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'template' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_TEMPLATE_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'templates',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_TEMPLATE_DESCRIPTION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'templates',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'a5ef0756-5ecd-49ae-a36c-058b90758988',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'dynamic_get' => [
				'name' => 'dynamic_get',
				'label' => 'COM_COMPONENTBUILDER_TEMPLATE_DYNAMIC_GET_LABEL',
				'type' => 'dynamicget',
				'title' => false,
				'list' => 'templates',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5087f379-710b-4e76-9b9d-86b842933991',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_dynamic_get',
					'component' => 'com_componentbuilder',
					'entity' => 'dynamic_get',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_TEMPLATE_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'templates',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_view' => [
				'name' => 'php_view',
				'label' => 'COM_COMPONENTBUILDER_TEMPLATE_PHP_VIEW_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'templates',
				'store' => 'base64',
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e27df414-cf8a-45c3-bc38-cc48e6146fef',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_view' => [
				'name' => 'add_php_view',
				'label' => 'COM_COMPONENTBUILDER_TEMPLATE_ADD_PHP_VIEW_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'templates',
				'store' => NULL,
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '8b09aaf9-546c-4f03-8b1f-3170ba4b5491',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'template' => [
				'name' => 'template',
				'label' => 'COM_COMPONENTBUILDER_TEMPLATE_TEMPLATE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'templates',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '7b57c7ca-f4e8-4358-948a-987b1f36b3fb',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'snippet' => [
				'name' => 'snippet',
				'label' => 'COM_COMPONENTBUILDER_TEMPLATE_SNIPPET_LABEL',
				'type' => 'snippets',
				'title' => false,
				'list' => 'templates',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'fc4bc2f3-1a12-4484-88dc-273199fe3b63',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_snippet',
					'component' => 'com_componentbuilder',
					'entity' => 'snippet',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'libraries' => [
				'name' => 'libraries',
				'label' => 'COM_COMPONENTBUILDER_TEMPLATE_LIBRARIES_LABEL',
				'type' => 'libraries',
				'title' => false,
				'list' => 'templates',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'f17742b9-0096-4b03-b981-f0dcd3f7e8c1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_library',
					'component' => 'com_componentbuilder',
					'entity' => 'library',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'alias' => [
				'name' => 'alias',
				'label' => 'COM_COMPONENTBUILDER_TEMPLATE_ALIAS_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'templates',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '477832a1-7a25-4774-ad1a-4ff69ed4b374',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'layout' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_LAYOUT_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'layouts',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_LAYOUT_DESCRIPTION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'layouts',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'a5ef0756-5ecd-49ae-a36c-058b90758988',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'dynamic_get' => [
				'name' => 'dynamic_get',
				'label' => 'COM_COMPONENTBUILDER_LAYOUT_DYNAMIC_GET_LABEL',
				'type' => 'dynamicget',
				'title' => false,
				'list' => 'layouts',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5087f379-710b-4e76-9b9d-86b842933991',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_dynamic_get',
					'component' => 'com_componentbuilder',
					'entity' => 'dynamic_get',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'snippet' => [
				'name' => 'snippet',
				'label' => 'COM_COMPONENTBUILDER_LAYOUT_SNIPPET_LABEL',
				'type' => 'snippets',
				'title' => false,
				'list' => 'layouts',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'fc4bc2f3-1a12-4484-88dc-273199fe3b63',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_snippet',
					'component' => 'com_componentbuilder',
					'entity' => 'snippet',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_LAYOUT_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'layouts',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_view' => [
				'name' => 'php_view',
				'label' => 'COM_COMPONENTBUILDER_LAYOUT_PHP_VIEW_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'layouts',
				'store' => 'base64',
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e27df414-cf8a-45c3-bc38-cc48e6146fef',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_view' => [
				'name' => 'add_php_view',
				'label' => 'COM_COMPONENTBUILDER_LAYOUT_ADD_PHP_VIEW_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'layouts',
				'store' => NULL,
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '8b09aaf9-546c-4f03-8b1f-3170ba4b5491',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'layout' => [
				'name' => 'layout',
				'label' => 'COM_COMPONENTBUILDER_LAYOUT_LAYOUT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'layouts',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '7c9289e1-b326-4596-9dd8-f6ebe861a0ef',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'libraries' => [
				'name' => 'libraries',
				'label' => 'COM_COMPONENTBUILDER_LAYOUT_LIBRARIES_LABEL',
				'type' => 'libraries',
				'title' => false,
				'list' => 'layouts',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'f17742b9-0096-4b03-b981-f0dcd3f7e8c1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_library',
					'component' => 'com_componentbuilder',
					'entity' => 'library',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'alias' => [
				'name' => 'alias',
				'label' => 'COM_COMPONENTBUILDER_LAYOUT_ALIAS_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'layouts',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '477832a1-7a25-4774-ad1a-4ff69ed4b374',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'dynamic_get' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Main',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'main_source' => [
				'name' => 'main_source',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_MAIN_SOURCE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Main',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '1709c1a3-b26a-4e78-82f9-ff05a08738c9',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'gettype' => [
				'name' => 'gettype',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_GETTYPE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Main',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '5377a931-8a17-4eed-92dd-8a8808408a5a',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_calculation' => [
				'name' => 'php_calculation',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_PHP_CALCULATION_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'base64',
				'tab_name' => 'Abacus',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '9b6768ae-8d6a-461e-9b41-61d49431a401',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_router_parse' => [
				'name' => 'php_router_parse',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_PHP_ROUTER_PARSE_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'base64',
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '88ffbef3-064c-4f87-997a-30af1c88f2ec',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_after_getitems' => [
				'name' => 'add_php_after_getitems',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_ADD_PHP_AFTER_GETITEMS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '01ae4944-ceba-4c21-a02d-38222e1a3733',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_router_parse' => [
				'name' => 'add_php_router_parse',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_ADD_PHP_ROUTER_PARSE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'd4b30bcc-414d-403b-9627-037bcbbd1c6b',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'view_selection' => [
				'name' => 'view_selection',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_VIEW_SELECTION_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Main',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '14f4d49c-7e54-494e-b103-eaccb6428ee8',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_php_before_getitems' => [
				'name' => 'add_php_before_getitems',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_ADD_PHP_BEFORE_GETITEMS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'b7aaa1d0-4577-46eb-9d09-b4c1104a02a8',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_before_getitem' => [
				'name' => 'add_php_before_getitem',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_ADD_PHP_BEFORE_GETITEM_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'bd40a792-2b63-446a-af75-0510f8c7cc2c',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_php_after_getitem' => [
				'name' => 'add_php_after_getitem',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_ADD_PHP_AFTER_GETITEM_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'c1754272-da91-4a17-9567-c2566552cdc9',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'db_table_main' => [
				'name' => 'db_table_main',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_DB_TABLE_MAIN_LABEL',
				'type' => 'dbtables',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Main',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '4c744096-3d1c-406d-87f4-71169002e85f',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 2,
					'table' => '',
					'component' => '',
					'entity' => '',
					'value' => '',
					'key' => '',
				],
			],
			'php_custom_get' => [
				'name' => 'php_custom_get',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_PHP_CUSTOM_GET_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'base64',
				'tab_name' => 'Main',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '8aba2f06-8fd1-4256-add5-1e233e581315',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'plugin_events' => [
				'name' => 'plugin_events',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_PLUGIN_EVENTS_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'json',
				'tab_name' => 'Main',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '13637273-b286-42f5-b092-90e507484751',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'db_selection' => [
				'name' => 'db_selection',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_DB_SELECTION_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Main',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '661c16f9-71be-41e2-8428-efd5509e6291',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'view_table_main' => [
				'name' => 'view_table_main',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_VIEW_TABLE_MAIN_LABEL',
				'type' => 'adminviews',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Main',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'f398a2ca-122e-47ad-8e4e-3ee2ac30bf26',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_admin_view',
					'component' => 'com_componentbuilder',
					'entity' => 'admin_view',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'add_php_getlistquery' => [
				'name' => 'add_php_getlistquery',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_ADD_PHP_GETLISTQUERY_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f2581140-5d9c-4c29-b511-e12a826d15da',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'join_db_table' => [
				'name' => 'join_db_table',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_JOIN_DB_TABLE_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'json',
				'tab_name' => 'Joint',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '6c75d3ff-93aa-493b-b25c-733e78408fa2',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'db_table' => [
						'name' => 'db_table',
						'type' => 'dbtables',
						'link' => [
							'type' => 2,
							'table' => '',
							'component' => '',
							'entity' => '',
							'value' => '',
							'key' => '',
						],
					],
					'row_type' => [
						'name' => 'row_type',
						'type' => 'list',
						'link' => NULL,
					],
					'as' => [
						'name' => 'as',
						'type' => 'list',
						'link' => NULL,
					],
					'type' => [
						'name' => 'type',
						'type' => 'list',
						'link' => NULL,
					],
					'on_field' => [
						'name' => 'on_field',
						'type' => 'text',
						'link' => NULL,
					],
					'operator' => [
						'name' => 'operator',
						'type' => 'list',
						'link' => NULL,
					],
					'join_field' => [
						'name' => 'join_field',
						'type' => 'text',
						'link' => NULL,
					],
					'selection' => [
						'name' => 'selection',
						'type' => 'textarea',
						'link' => NULL,
					],
				],
			],
			'select_all' => [
				'name' => 'select_all',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_SELECT_ALL_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Main',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'fbb918ff-70bf-41ee-b1fd-110e38cbdba0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_before_getitem' => [
				'name' => 'php_before_getitem',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_PHP_BEFORE_GETITEM_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'base64',
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'c3bb9074-505a-4085-878b-f7ad3d5522a6',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'getcustom' => [
				'name' => 'getcustom',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_GETCUSTOM_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Main',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '6c486f71-abfd-49c7-91e2-165f594fa658',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_after_getitem' => [
				'name' => 'php_after_getitem',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_PHP_AFTER_GETITEM_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'base64',
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'fc0dcf64-f6a8-48d7-8ed7-6ecea942cf21',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'pagination' => [
				'name' => 'pagination',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_PAGINATION_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Main',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '1',
					'GUID' => 'b99a8df9-27ce-4502-9f44-863799cd2cc9',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'php_getlistquery' => [
				'name' => 'php_getlistquery',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_PHP_GETLISTQUERY_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'base64',
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'b9c064cb-4266-4217-95b7-d2956e6bfefb',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'php_before_getitems' => [
				'name' => 'php_before_getitems',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_PHP_BEFORE_GETITEMS_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'base64',
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '71680787-5da5-4ab8-afdf-1e109556743b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'filter' => [
				'name' => 'filter',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_FILTER_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'json',
				'tab_name' => 'Tweak',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'bf2293c7-5c8b-4b77-a880-c63ca4777e3e',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'filter_type' => [
						'name' => 'filter_type',
						'type' => 'list',
						'link' => NULL,
					],
					'state_key' => [
						'name' => 'state_key',
						'type' => 'text',
						'link' => NULL,
					],
					'operator' => [
						'name' => 'operator',
						'type' => 'list',
						'link' => NULL,
					],
					'table_key' => [
						'name' => 'table_key',
						'type' => 'text',
						'link' => NULL,
					],
					'empty' => [
						'name' => 'empty',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'php_after_getitems' => [
				'name' => 'php_after_getitems',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_PHP_AFTER_GETITEMS_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'base64',
				'tab_name' => 'Custom Script',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '1ae39469-cd74-4d82-84af-fec8af28be61',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'where' => [
				'name' => 'where',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_WHERE_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'json',
				'tab_name' => 'Tweak',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '81466df0-3571-45cf-b520-cee2a9f5e03b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'table_key' => [
						'name' => 'table_key',
						'type' => 'text',
						'link' => NULL,
					],
					'operator' => [
						'name' => 'operator',
						'type' => 'list',
						'link' => NULL,
					],
					'value_key' => [
						'name' => 'value_key',
						'type' => 'text',
						'link' => NULL,
					],
				],
			],
			'order' => [
				'name' => 'order',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_ORDER_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'json',
				'tab_name' => 'Tweak',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '007dc9f8-9b06-4211-83c9-4134b23d6561',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'table_key' => [
						'name' => 'table_key',
						'type' => 'text',
						'link' => NULL,
					],
					'direction' => [
						'name' => 'direction',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'addcalculation' => [
				'name' => 'addcalculation',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_ADDCALCULATION_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'Abacus',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '6e7bb46d-fda5-46b8-b08e-31a2e6884e24',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'group' => [
				'name' => 'group',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_GROUP_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'json',
				'tab_name' => 'Tweak',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'f37d69ac-ba75-4b2f-8793-91b77b86751f',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'table_key' => [
						'name' => 'table_key',
						'type' => 'text',
						'link' => NULL,
					],
				],
			],
			'global' => [
				'name' => 'global',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_GLOBAL_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'json',
				'tab_name' => 'Tweak',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'fa2f73d6-8427-47de-abd8-4290da426b49',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'name' => [
						'name' => 'name',
						'type' => 'text',
						'link' => NULL,
					],
					'key' => [
						'name' => 'key',
						'type' => 'text',
						'link' => NULL,
					],
					'as' => [
						'name' => 'as',
						'type' => 'list',
						'link' => NULL,
					],
					'type' => [
						'name' => 'type',
						'type' => 'radio',
						'link' => NULL,
					],
				],
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'join_view_table' => [
				'name' => 'join_view_table',
				'label' => 'COM_COMPONENTBUILDER_DYNAMIC_GET_JOIN_VIEW_TABLE_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'dynamic_gets',
				'store' => 'json',
				'tab_name' => 'Joint',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '79ab7f6c-a6b5-4bad-bcdd-c113d49db876',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'view_table' => [
						'name' => 'view_table',
						'type' => 'adminviews',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_admin_view',
							'component' => 'com_componentbuilder',
							'entity' => 'admin_view',
							'value' => 'system_name',
							'key' => 'guid',
						],
					],
					'row_type' => [
						'name' => 'row_type',
						'type' => 'list',
						'link' => NULL,
					],
					'as' => [
						'name' => 'as',
						'type' => 'list',
						'link' => NULL,
					],
					'type' => [
						'name' => 'type',
						'type' => 'list',
						'link' => NULL,
					],
					'on_field' => [
						'name' => 'on_field',
						'type' => 'text',
						'link' => NULL,
					],
					'operator' => [
						'name' => 'operator',
						'type' => 'list',
						'link' => NULL,
					],
					'join_field' => [
						'name' => 'join_field',
						'type' => 'text',
						'link' => NULL,
					],
					'selection' => [
						'name' => 'selection',
						'type' => 'textarea',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'custom_code' => [
			'system_name' => [
				'name' => 'system_name',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_CODE_SYSTEM_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'custom_codes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'acfe906b-6e61-4f94-ae66-359e4bc3e4cc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'path' => [
				'name' => 'path',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_CODE_PATH_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'custom_codes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'b7942a37-45cc-4005-b4b5-e716cbf176f9',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'target' => [
				'name' => 'target',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_CODE_TARGET_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'custom_codes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'b4fce451-7c6d-4cf9-8b1c-f8be6dbcec5b',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'type' => [
				'name' => 'type',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_CODE_TYPE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'custom_codes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'd1c48683-2655-4e49-9f13-6b25f2b0a0d0',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'comment_type' => [
				'name' => 'comment_type',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_CODE_COMMENT_TYPE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'custom_codes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '1',
					'GUID' => '176c63c6-65b1-4a91-97f2-db4d1e36911a',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'joomla_version' => [
				'name' => 'joomla_version',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_CODE_JOOMLA_VERSION_LABEL',
				'type' => 'number',
				'title' => false,
				'list' => 'custom_codes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(11)',
					'default' => '3',
					'GUID' => 'a6c87660-f1fe-490d-b1c5-9755e6db1e19',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'function_name' => [
				'name' => 'function_name',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_CODE_FUNCTION_NAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'custom_codes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '3a3c43e6-18c2-4fc6-aebe-0ed8d5c6416f',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'code' => [
				'name' => 'code',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_CODE_CODE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'custom_codes',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '777b846c-f3f4-4eb3-a73a-a460d28976dc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'hashendtarget' => [
				'name' => 'hashendtarget',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_CODE_HASHENDTARGET_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'custom_codes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'GUID' => '3347f841-b8e8-4f52-acbd-0bada696e540',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'to_line' => [
				'name' => 'to_line',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_CODE_TO_LINE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'custom_codes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(100)',
					'default' => '',
					'GUID' => 'ebbfce08-7d5a-416a-aad2-b23b78bd926b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'from_line' => [
				'name' => 'from_line',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_CODE_FROM_LINE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'custom_codes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(100)',
					'default' => '',
					'GUID' => '82e196f5-cc38-4045-8be3-c1067b23df72',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'hashtarget' => [
				'name' => 'hashtarget',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_CODE_HASHTARGET_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'custom_codes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'GUID' => 'e0272089-9156-409b-8444-61e2e3e755bf',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'component' => [
				'name' => 'component',
				'label' => 'COM_COMPONENTBUILDER_CUSTOM_CODE_COMPONENT_LABEL',
				'type' => 'joomlacomponent',
				'title' => false,
				'list' => 'custom_codes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '04886e45-7821-40a6-ae2b-2f46c6b45906',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'class_extends' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_CLASS_EXTENDS_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'class_extendings',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'extension_type' => [
				'name' => 'extension_type',
				'label' => 'COM_COMPONENTBUILDER_CLASS_EXTENDS_EXTENSION_TYPE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'class_extendings',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'GUID' => '7e059ab7-d79b-4218-9488-6f38d85d17a0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_CLASS_EXTENDS_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'class_extendings',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'head' => [
				'name' => 'head',
				'label' => 'COM_COMPONENTBUILDER_CLASS_EXTENDS_HEAD_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'class_extendings',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'e4a3ccf7-e259-4b72-ad4c-216b15d6d4f7',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'comment' => [
				'name' => 'comment',
				'label' => 'COM_COMPONENTBUILDER_CLASS_EXTENDS_COMMENT_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'class_extendings',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '3d80f045-6262-43c8-9b43-658093f6a057',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'class_property' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_CLASS_PROPERTY_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'class_properties',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'visibility' => [
				'name' => 'visibility',
				'label' => 'COM_COMPONENTBUILDER_CLASS_PROPERTY_VISIBILITY_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'class_properties',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '7222c742-0092-439c-84de-1973e4e2b180',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'extension_type' => [
				'name' => 'extension_type',
				'label' => 'COM_COMPONENTBUILDER_CLASS_PROPERTY_EXTENSION_TYPE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'class_properties',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'GUID' => '7e059ab7-d79b-4218-9488-6f38d85d17a0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_CLASS_PROPERTY_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'class_properties',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'comment' => [
				'name' => 'comment',
				'label' => 'COM_COMPONENTBUILDER_CLASS_PROPERTY_COMMENT_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'class_properties',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'dd400eb4-8762-4ab5-bc4e-fa1feacc4dfd',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'joomla_plugin_group' => [
				'name' => 'joomla_plugin_group',
				'label' => 'COM_COMPONENTBUILDER_CLASS_PROPERTY_JOOMLA_PLUGIN_GROUP_LABEL',
				'type' => 'joomlaplugingroups',
				'title' => false,
				'list' => 'class_properties',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'd23a041c-6897-4cef-8941-a26a2114a865',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_plugin_group',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_plugin_group',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'default' => [
				'name' => 'default',
				'label' => 'COM_COMPONENTBUILDER_CLASS_PROPERTY_DEFAULT_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'class_properties',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '73ef5c00-c8c5-4cb7-9f69-6aed98346818',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'class_method' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_CLASS_METHOD_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'class_methods',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'visibility' => [
				'name' => 'visibility',
				'label' => 'COM_COMPONENTBUILDER_CLASS_METHOD_VISIBILITY_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'class_methods',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '7222c742-0092-439c-84de-1973e4e2b180',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'extension_type' => [
				'name' => 'extension_type',
				'label' => 'COM_COMPONENTBUILDER_CLASS_METHOD_EXTENSION_TYPE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'class_methods',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'GUID' => '7e059ab7-d79b-4218-9488-6f38d85d17a0',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_CLASS_METHOD_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'class_methods',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'code' => [
				'name' => 'code',
				'label' => 'COM_COMPONENTBUILDER_CLASS_METHOD_CODE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'class_methods',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'abaff633-2a9f-4be1-b057-241f657953a1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'comment' => [
				'name' => 'comment',
				'label' => 'COM_COMPONENTBUILDER_CLASS_METHOD_COMMENT_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'class_methods',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'dd400eb4-8762-4ab5-bc4e-fa1feacc4dfd',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'joomla_plugin_group' => [
				'name' => 'joomla_plugin_group',
				'label' => 'COM_COMPONENTBUILDER_CLASS_METHOD_JOOMLA_PLUGIN_GROUP_LABEL',
				'type' => 'joomlaplugingroups',
				'title' => false,
				'list' => 'class_methods',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'd23a041c-6897-4cef-8941-a26a2114a865',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_plugin_group',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_plugin_group',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'arguments' => [
				'name' => 'arguments',
				'label' => 'COM_COMPONENTBUILDER_CLASS_METHOD_ARGUMENTS_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'class_methods',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'd2a2f6cf-1689-438b-b2c2-b244b8cbf078',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'placeholder' => [
			'target' => [
				'name' => 'target',
				'label' => 'COM_COMPONENTBUILDER_PLACEHOLDER_TARGET_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'placeholders',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '65b533a8-0d70-44af-a9d1-785983eb4ef3',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'value' => [
				'name' => 'value',
				'label' => 'COM_COMPONENTBUILDER_PLACEHOLDER_VALUE_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'placeholders',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'b14c6496-2aaf-4198-8fc1-cf034525fb91',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'library' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'libraries',
				'store' => NULL,
				'tab_name' => 'Behaviour',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'target' => [
				'name' => 'target',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_TARGET_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'libraries',
				'store' => NULL,
				'tab_name' => 'Behaviour',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '1',
					'GUID' => '07c71471-04cd-4217-a63e-4ca97d880950',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'how' => [
				'name' => 'how',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_HOW_LABEL',
				'type' => 'filebehaviour',
				'title' => false,
				'list' => 'libraries',
				'store' => NULL,
				'tab_name' => 'Behaviour',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '1',
					'GUID' => '0051004f-b0ec-47c4-bfdb-0d8e76adbbb8',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 2,
					'table' => '',
					'component' => '',
					'entity' => '',
					'value' => '',
					'key' => '',
				],
			],
			'type' => [
				'name' => 'type',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_TYPE_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'libraries',
				'store' => NULL,
				'tab_name' => 'Behaviour',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f2940a48-f104-4c11-86ef-ed6456a1211c',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_DESCRIPTION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'libraries',
				'store' => NULL,
				'tab_name' => 'Behaviour',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '09b25ca3-6c9f-4b6d-ad80-e0989c221289',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'libraries' => [
				'name' => 'libraries',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_LIBRARIES_LABEL',
				'type' => 'librariesx',
				'title' => false,
				'list' => 'libraries',
				'store' => 'json',
				'tab_name' => 'Behaviour',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '7e1418d5-5515-49f6-99ed-166365b9b963',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_library',
					'component' => 'com_componentbuilder',
					'entity' => 'library',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'php_setdocument' => [
				'name' => 'php_setdocument',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_PHP_SETDOCUMENT_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'libraries',
				'store' => 'base64',
				'tab_name' => 'Behaviour',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'b70529a1-30bb-42c0-822f-802c23bd616b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'addconditions' => [
				'name' => 'addconditions',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_ADDCONDITIONS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'libraries',
				'store' => 'json',
				'tab_name' => 'Behaviour',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '48d69b71-6eba-480a-b17b-c67243f02d06',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'file' => [
						'name' => 'file',
						'type' => 'libraryfiles',
						'link' => [
							'type' => 2,
							'table' => '',
							'component' => '',
							'entity' => '',
							'value' => '',
							'key' => '',
						],
					],
					'target_behavior' => [
						'name' => 'target_behavior',
						'type' => 'list',
						'link' => NULL,
					],
					'target_relation' => [
						'name' => 'target_relation',
						'type' => 'list',
						'link' => NULL,
					],
					'option_field' => [
						'name' => 'option_field',
						'type' => 'libconfigfield',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_field',
							'component' => 'com_componentbuilder',
							'entity' => 'field',
							'value' => 'name',
							'key' => 'guid',
						],
					],
					'option_behaviour' => [
						'name' => 'option_behaviour',
						'type' => 'list',
						'link' => NULL,
					],
					'field_options' => [
						'name' => 'field_options',
						'type' => 'textarea',
						'link' => NULL,
					],
				],
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'libraries',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'snippet' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'snippets',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'url' => [
				'name' => 'url',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_URL_LABEL',
				'type' => 'url',
				'title' => false,
				'list' => 'snippets',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'd29d6854-bb4a-4fa2-83a1-5a1b9c9a0eaf',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'type' => [
				'name' => 'type',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_TYPE_LABEL',
				'type' => 'snippettype',
				'title' => false,
				'list' => 'snippets',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '0cc69ef6-eb4d-4a0d-ad40-84bc1845e48f',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_snippet_type',
					'component' => 'com_componentbuilder',
					'entity' => 'snippet_type',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'heading' => [
				'name' => 'heading',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_HEADING_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'snippets',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'fc671dc8-a9be-43d9-b42a-671750ad869d',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'library' => [
				'name' => 'library',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_LIBRARY_LABEL',
				'type' => 'library',
				'title' => false,
				'list' => 'snippets',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '8b97aaa3-28ae-45f1-80ad-6f37116264cf',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_library',
					'component' => 'com_componentbuilder',
					'entity' => 'library',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'snippets',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'contributor_email' => [
				'name' => 'contributor_email',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_CONTRIBUTOR_EMAIL_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'snippets',
				'store' => NULL,
				'tab_name' => 'Contributor',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '1e0608ea-045b-494b-aac6-060dc76da782',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'contributor_name' => [
				'name' => 'contributor_name',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_CONTRIBUTOR_NAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'snippets',
				'store' => NULL,
				'tab_name' => 'Contributor',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '6803fdeb-c251-4588-85c9-d0e8636d18ab',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'contributor_website' => [
				'name' => 'contributor_website',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_CONTRIBUTOR_WEBSITE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'snippets',
				'store' => NULL,
				'tab_name' => 'Contributor',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '3a1db9fd-204e-4693-bcf7-2084d26c1013',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'contributor_company' => [
				'name' => 'contributor_company',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_CONTRIBUTOR_COMPANY_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'snippets',
				'store' => NULL,
				'tab_name' => 'Contributor',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'cef2d060-95cf-409a-92c9-02de0c164640',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'snippet' => [
				'name' => 'snippet',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_SNIPPET_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'snippets',
				'store' => 'base64',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '144a1f95-97d5-42b4-be3e-39f21215780e',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'usage' => [
				'name' => 'usage',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_USAGE_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'snippets',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '155d8238-a2cf-446c-91a5-c8322ee1d890',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_DESCRIPTION_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'snippets',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '749a9917-90c3-49c4-9e72-aa33b0683a87',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'validation_rule' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_VALIDATION_RULE_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'validation_rules',
				'store' => NULL,
				'tab_name' => 'Extends FormRule',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '9a98c57f-bf01-49b0-91e3-e72e1b9551f1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'short_description' => [
				'name' => 'short_description',
				'label' => 'COM_COMPONENTBUILDER_VALIDATION_RULE_SHORT_DESCRIPTION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'validation_rules',
				'store' => NULL,
				'tab_name' => 'Extends FormRule',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '46503007-43d9-485f-8cd0-8427a6a8bef2',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'inherit' => [
				'name' => 'inherit',
				'label' => 'COM_COMPONENTBUILDER_VALIDATION_RULE_INHERIT_LABEL',
				'type' => 'existingvalidationrules',
				'title' => false,
				'list' => 'validation_rules',
				'store' => NULL,
				'tab_name' => 'Extends FormRule',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => '419fa655-e64a-4e39-b124-29eeb3198bc7',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_validation_rule',
					'component' => 'com_componentbuilder',
					'entity' => 'validation_rule',
					'value' => 'name',
					'key' => 'id',
				],
			],
			'php' => [
				'name' => 'php',
				'label' => 'COM_COMPONENTBUILDER_VALIDATION_RULE_PHP_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'validation_rules',
				'store' => 'base64',
				'tab_name' => 'Extends FormRule',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '24ed6b0e-c86c-4131-91a7-5205ac931ad5',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'field' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_FIELD_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Set Properties',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'fieldtype' => [
				'name' => 'fieldtype',
				'label' => 'COM_COMPONENTBUILDER_FIELD_FIELDTYPE_LABEL',
				'type' => 'fieldtypes',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Set Properties',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '91a4f538-1383-4ae4-85d8-55ddada7d6cd',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_fieldtype',
					'component' => 'com_componentbuilder',
					'entity' => 'fieldtype',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'datatype' => [
				'name' => 'datatype',
				'label' => 'COM_COMPONENTBUILDER_FIELD_DATATYPE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Database',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => 'e41e74b2-a265-4269-8ec6-1d890c893798',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'indexes' => [
				'name' => 'indexes',
				'label' => 'COM_COMPONENTBUILDER_FIELD_INDEXES_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Database',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => '63ff758d-a866-4d37-bc2a-a7432bd9bb44',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'null_switch' => [
				'name' => 'null_switch',
				'label' => 'COM_COMPONENTBUILDER_FIELD_NULL_SWITCH_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Database',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => '6a3d21ad-7cb1-4456-a623-20aa71ba0b12',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'store' => [
				'name' => 'store',
				'label' => 'COM_COMPONENTBUILDER_FIELD_STORE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Database',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'GUID' => '48b78ab1-bfb0-44eb-9ec8-4f5cdbf4a32c',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'catid' => [
				'name' => 'catid',
				'label' => 'COM_COMPONENTBUILDER_FIELD_CATID_LABEL',
				'type' => 'category',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Set Properties',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'GUID' => 'ef722d2d-0fa0-4aa7-92d2-7783ec1c9f7f',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'on_get_model_field' => [
				'name' => 'on_get_model_field',
				'label' => 'COM_COMPONENTBUILDER_FIELD_ON_GET_MODEL_FIELD_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'fields',
				'store' => 'base64',
				'tab_name' => 'Database',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'dc69b6d4-2e6f-4f2d-86d9-0fb6534164ca',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'on_save_model_field' => [
				'name' => 'on_save_model_field',
				'label' => 'COM_COMPONENTBUILDER_FIELD_ON_SAVE_MODEL_FIELD_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'fields',
				'store' => 'base64',
				'tab_name' => 'Database',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '0f0ef89c-e305-408d-ac95-6a86efe28c5a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'initiator_on_get_model' => [
				'name' => 'initiator_on_get_model',
				'label' => 'COM_COMPONENTBUILDER_FIELD_INITIATOR_ON_GET_MODEL_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'fields',
				'store' => 'base64',
				'tab_name' => 'Database',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '9d32c0a5-e010-44f2-8b53-3372c87da08d',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'xml' => [
				'name' => 'xml',
				'label' => '',
				'type' => 'hidden',
				'title' => false,
				'list' => 'fields',
				'store' => 'json',
				'tab_name' => 'Type Info',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '83a3c115-1a4f-44a7-9572-e91043acdbaa',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'datalenght' => [
				'name' => 'datalenght',
				'label' => 'COM_COMPONENTBUILDER_FIELD_DATALENGHT_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Database',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => 'b3ea0d94-1029-41d8-9f6b-bcbec6b715f4',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'javascript_view_footer' => [
				'name' => 'javascript_view_footer',
				'label' => 'COM_COMPONENTBUILDER_FIELD_JAVASCRIPT_VIEW_FOOTER_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'fields',
				'store' => 'base64',
				'tab_name' => 'Scripts',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'a99c1f40-4b41-4167-ad29-85133375aa7a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'css_views' => [
				'name' => 'css_views',
				'label' => 'COM_COMPONENTBUILDER_FIELD_CSS_VIEWS_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'fields',
				'store' => 'base64',
				'tab_name' => 'Scripts',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '63d5585f-c9bd-485d-9c1d-4a29a4fbf4b2',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'css_view' => [
				'name' => 'css_view',
				'label' => 'COM_COMPONENTBUILDER_FIELD_CSS_VIEW_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'fields',
				'store' => 'base64',
				'tab_name' => 'Scripts',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '1ad12721-9ee9-4623-98d2-071ba013a373',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'datadefault_other' => [
				'name' => 'datadefault_other',
				'label' => 'COM_COMPONENTBUILDER_FIELD_DATADEFAULT_OTHER_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Database',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => '6b2277f3-44e1-4bbb-8e55-3315bdba736e',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'datadefault' => [
				'name' => 'datadefault',
				'label' => 'COM_COMPONENTBUILDER_FIELD_DATADEFAULT_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Database',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => '6ba76a0f-55a4-49ce-911a-0a3f05674bef',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'datalenght_other' => [
				'name' => 'datalenght_other',
				'label' => 'COM_COMPONENTBUILDER_FIELD_DATALENGHT_OTHER_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Database',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => 'fd7ff7b0-a8e9-476c-8e9d-ac4ca338c589',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'javascript_views_footer' => [
				'name' => 'javascript_views_footer',
				'label' => 'COM_COMPONENTBUILDER_FIELD_JAVASCRIPT_VIEWS_FOOTER_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'fields',
				'store' => 'base64',
				'tab_name' => 'Scripts',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '34bf9926-34a4-49a3-a29e-b602b1e5f9b2',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'add_css_view' => [
				'name' => 'add_css_view',
				'label' => 'COM_COMPONENTBUILDER_FIELD_ADD_CSS_VIEW_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Scripts',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '6746d3ba-b120-4918-8bba-2a14260e99c8',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_css_views' => [
				'name' => 'add_css_views',
				'label' => 'COM_COMPONENTBUILDER_FIELD_ADD_CSS_VIEWS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Scripts',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'a2098911-926a-42b6-8498-d8a453c34d93',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_javascript_view_footer' => [
				'name' => 'add_javascript_view_footer',
				'label' => 'COM_COMPONENTBUILDER_FIELD_ADD_JAVASCRIPT_VIEW_FOOTER_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Scripts',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '9405da44-900c-4f7c-823d-734a68f5d6f6',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'add_javascript_views_footer' => [
				'name' => 'add_javascript_views_footer',
				'label' => 'COM_COMPONENTBUILDER_FIELD_ADD_JAVASCRIPT_VIEWS_FOOTER_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'Scripts',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '70995b9c-d0a9-4a57-b59b-54d39c379e66',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'initiator_on_save_model' => [
				'name' => 'initiator_on_save_model',
				'label' => 'COM_COMPONENTBUILDER_FIELD_INITIATOR_ON_SAVE_MODEL_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'fields',
				'store' => 'base64',
				'tab_name' => 'Database',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '49a285a3-2c16-4846-9dbf-94c9cc6496c1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_FIELD_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'fields',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'fieldtype' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'store' => [
				'name' => 'store',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_STORE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'Database (defaults)',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'GUID' => '48b78ab1-bfb0-44eb-9ec8-4f5cdbf4a32c',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'null_switch' => [
				'name' => 'null_switch',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_NULL_SWITCH_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'Database (defaults)',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => '6a3d21ad-7cb1-4456-a623-20aa71ba0b12',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'indexes' => [
				'name' => 'indexes',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_INDEXES_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'Database (defaults)',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => '63ff758d-a866-4d37-bc2a-a7432bd9bb44',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'datadefault_other' => [
				'name' => 'datadefault_other',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_DATADEFAULT_OTHER_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'Database (defaults)',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => '6b2277f3-44e1-4bbb-8e55-3315bdba736e',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'datadefault' => [
				'name' => 'datadefault',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_DATADEFAULT_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'Database (defaults)',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => '6ba76a0f-55a4-49ce-911a-0a3f05674bef',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'short_description' => [
				'name' => 'short_description',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_SHORT_DESCRIPTION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '46503007-43d9-485f-8cd0-8427a6a8bef2',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'datatype' => [
				'name' => 'datatype',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_DATATYPE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'Database (defaults)',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => 'e41e74b2-a265-4269-8ec6-1d890c893798',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'has_defaults' => [
				'name' => 'has_defaults',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_HAS_DEFAULTS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'Database (defaults)',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'ec2ec4a2-a32f-4407-b318-46b5780774e2',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'properties' => [
				'name' => 'properties',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_PROPERTIES_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'ddb8f57f-77c8-4cad-a8cb-0cdc799ce4a1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'name' => [
						'name' => 'name',
						'type' => 'text',
						'link' => NULL,
					],
					'example' => [
						'name' => 'example',
						'type' => 'textarea',
						'link' => NULL,
					],
					'adjustable' => [
						'name' => 'adjustable',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'mandatory' => [
						'name' => 'mandatory',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'translatable' => [
						'name' => 'translatable',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'description' => [
						'name' => 'description',
						'type' => 'textarea',
						'link' => NULL,
					],
				],
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_DESCRIPTION_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '749a9917-90c3-49c4-9e72-aa33b0683a87',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'datalenght' => [
				'name' => 'datalenght',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_DATALENGHT_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'Database (defaults)',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => 'b3ea0d94-1029-41d8-9f6b-bcbec6b715f4',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'datalenght_other' => [
				'name' => 'datalenght_other',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_DATALENGHT_OTHER_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'Database (defaults)',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => 'fd7ff7b0-a8e9-476c-8e9d-ac4ca338c589',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'catid' => [
				'name' => 'catid',
				'label' => 'COM_COMPONENTBUILDER_FIELDTYPE_CATID_LABEL',
				'type' => 'category',
				'title' => false,
				'list' => 'fieldtypes',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'INT(11)',
					'default' => '0',
					'GUID' => '7cc4ab46-0d77-4ef8-a2a7-d5b656546a30',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'language_translation' => [
			'source' => [
				'name' => 'source',
				'label' => 'COM_COMPONENTBUILDER_LANGUAGE_TRANSLATION_SOURCE_LABEL',
				'type' => 'textarea',
				'title' => true,
				'list' => 'language_translations',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'c2f5d193-ef76-422b-aae5-421cd0a4b22b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'plugins' => [
				'name' => 'plugins',
				'label' => 'COM_COMPONENTBUILDER_LANGUAGE_TRANSLATION_PLUGINS_LABEL',
				'type' => 'joomlaplugins',
				'title' => false,
				'list' => 'language_translations',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '6e1717a7-99b8-4101-a0b0-fcf2a5e1cfa0',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_plugin',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_plugin',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'modules' => [
				'name' => 'modules',
				'label' => 'COM_COMPONENTBUILDER_LANGUAGE_TRANSLATION_MODULES_LABEL',
				'type' => 'joomlamodules',
				'title' => false,
				'list' => 'language_translations',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'd3468164-df0a-4f0d-a679-1aa95ae408fe',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_module',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_module',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'components' => [
				'name' => 'components',
				'label' => 'COM_COMPONENTBUILDER_LANGUAGE_TRANSLATION_COMPONENTS_LABEL',
				'type' => 'joomlacomponents',
				'title' => false,
				'list' => 'language_translations',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '61990265-efe5-49d3-ae08-4bd0e8f3e4ce',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'translation' => [
				'name' => 'translation',
				'label' => 'COM_COMPONENTBUILDER_LANGUAGE_TRANSLATION_TRANSLATION_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'language_translations',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '36edbdce-b9b7-4b89-b2d9-03f91bb56019',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'translation' => [
						'name' => 'translation',
						'type' => 'textarea',
						'link' => NULL,
					],
					'language' => [
						'name' => 'language',
						'type' => 'lang',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_language',
							'component' => 'com_componentbuilder',
							'entity' => 'language',
							'value' => 'name',
							'key' => 'langtag',
						],
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'language' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_LANGUAGE_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'languages',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'langtag' => [
				'name' => 'langtag',
				'label' => 'COM_COMPONENTBUILDER_LANGUAGE_LANGTAG_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'languages',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(64)',
					'default' => '',
					'GUID' => '8fdf3640-8668-4818-be46-36c74f9e103e',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'server' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_SERVER_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'servers',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'protocol' => [
				'name' => 'protocol',
				'label' => 'COM_COMPONENTBUILDER_SERVER_PROTOCOL_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'servers',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'a5bb2b19-d773-465c-a7e3-ce6400735734',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'signature' => [
				'name' => 'signature',
				'label' => 'COM_COMPONENTBUILDER_SERVER_SIGNATURE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'servers',
				'store' => 'basic_encryption',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '59688b83-dca9-4491-aefb-39c18057394e',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'private_key' => [
				'name' => 'private_key',
				'label' => 'COM_COMPONENTBUILDER_SERVER_PRIVATE_KEY_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'servers',
				'store' => 'basic_encryption',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '29b8c31d-6e2a-4f8d-90c7-febfbb9cc16c',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'secret' => [
				'name' => 'secret',
				'label' => 'COM_COMPONENTBUILDER_SERVER_SECRET_LABEL',
				'type' => 'password',
				'title' => false,
				'list' => 'servers',
				'store' => 'basic_encryption',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'c6ccb2ba-4d74-487a-99a8-f2bfe40a23f1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'password' => [
				'name' => 'password',
				'label' => 'COM_COMPONENTBUILDER_SERVER_PASSWORD_LABEL',
				'type' => 'password',
				'title' => false,
				'list' => 'servers',
				'store' => 'basic_encryption',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '0a6939be-aabc-46df-b922-f083cf0c5bd4',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'private' => [
				'name' => 'private',
				'label' => 'COM_COMPONENTBUILDER_SERVER_PRIVATE_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'servers',
				'store' => 'basic_encryption',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '2c689678-6cef-40ef-97da-d59ab7d039d1',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'authentication' => [
				'name' => 'authentication',
				'label' => 'COM_COMPONENTBUILDER_SERVER_AUTHENTICATION_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'servers',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '452b22ac-adfc-4fca-9d82-5461806bf847',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'path' => [
				'name' => 'path',
				'label' => 'COM_COMPONENTBUILDER_SERVER_PATH_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'servers',
				'store' => 'basic_encryption',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '593fe848-22f3-45d1-8d8d-7824542a7381',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'port' => [
				'name' => 'port',
				'label' => 'COM_COMPONENTBUILDER_SERVER_PORT_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'servers',
				'store' => 'basic_encryption',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '5a64961d-8aa6-4de4-92e2-997199e5a707',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'host' => [
				'name' => 'host',
				'label' => 'COM_COMPONENTBUILDER_SERVER_HOST_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'servers',
				'store' => 'basic_encryption',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '3eed4f04-46cc-43e8-9712-b6e339e71241',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'username' => [
				'name' => 'username',
				'label' => 'COM_COMPONENTBUILDER_SERVER_USERNAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'servers',
				'store' => 'basic_encryption',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '41152cdd-e774-4edf-be9b-280d272d5712',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'repository' => [
			'system_name' => [
				'name' => 'system_name',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_SYSTEM_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'acfe906b-6e61-4f94-ae66-359e4bc3e4cc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'organisation' => [
				'name' => 'organisation',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_ORGANISATION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'eaaca317-58c7-4b72-9c20-166c12c04b67',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'repository' => [
				'name' => 'repository',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_REPOSITORY_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '4c224b6d-7b6b-4e5d-8df7-b96c26559476',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'target' => [
				'name' => 'target',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_TARGET_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(7)',
					'default' => '1',
					'GUID' => 'd22126a7-7e85-4a7a-94d0-6ba9ab194772',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'type' => [
				'name' => 'type',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_TYPE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '1',
					'GUID' => '3557698d-cfae-4a57-93fc-6030bac2768a',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'base' => [
				'name' => 'base',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_BASE_LABEL',
				'type' => 'url',
				'title' => false,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '0ca15427-abfa-4fac-8ce2-e70e5b8b2c0f',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'addplaceholders' => [
				'name' => 'addplaceholders',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_ADDPLACEHOLDERS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'repositories',
				'store' => 'json',
				'tab_name' => 'Placeholders',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'fb4e322b-46c0-47e5-b9eb-70c39456d80d',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'target' => [
						'name' => 'target',
						'type' => 'text',
						'link' => NULL,
					],
					'value' => [
						'name' => 'value',
						'type' => 'textarea',
						'link' => NULL,
					],
				],
			],
			'access_repo' => [
				'name' => 'access_repo',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_ACCESS_REPO_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '1',
					'GUID' => '5e9038cf-bf09-4ec7-aa1d-60df007b96e2',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'write_branch' => [
				'name' => 'write_branch',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_WRITE_BRANCH_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '8a5addab-5545-4c33-bc1a-7aad217e04c6',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'read_branch' => [
				'name' => 'read_branch',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_READ_BRANCH_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'ce236d83-959b-4d1c-a736-b1717bee29bc',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'author_email' => [
				'name' => 'author_email',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_AUTHOR_EMAIL_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '3dc12b2f-d1c9-4ae4-8573-19a6b7d78083',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'author_name' => [
				'name' => 'author_name',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_AUTHOR_NAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'c2bd3a42-bfc9-459f-b134-5302798ed0e4',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'token' => [
				'name' => 'token',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_TOKEN_LABEL',
				'type' => 'password',
				'title' => false,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'f8aa483a-71b9-46c1-a08e-0f546076fd78',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'username' => [
				'name' => 'username',
				'label' => 'COM_COMPONENTBUILDER_REPOSITORY_USERNAME_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'repositories',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'fcbab2f2-51bb-4f44-a5ac-a6266e187522',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'help_document' => [
			'title' => [
				'name' => 'title',
				'label' => 'COM_COMPONENTBUILDER_HELP_DOCUMENT_TITLE_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'help_documents',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => 'edda1847-ef2e-4a76-9763-c41c82b08c00',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'type' => [
				'name' => 'type',
				'label' => 'COM_COMPONENTBUILDER_HELP_DOCUMENT_TYPE_LABEL',
				'type' => 'list',
				'title' => false,
				'list' => 'help_documents',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '8487b70c-4a32-46ba-aacb-ea07a1e3d278',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'groups' => [
				'name' => 'groups',
				'label' => 'COM_COMPONENTBUILDER_HELP_DOCUMENT_GROUPS_LABEL',
				'type' => 'usergrouplist',
				'title' => false,
				'list' => 'help_documents',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '59a0cad6-26cc-43b6-8af8-2242eb595d76',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'location' => [
				'name' => 'location',
				'label' => 'COM_COMPONENTBUILDER_HELP_DOCUMENT_LOCATION_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'help_documents',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '0d06a4f6-acd5-4acc-8a3f-28813de178ac',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'admin_view' => [
				'name' => 'admin_view',
				'label' => 'COM_COMPONENTBUILDER_HELP_DOCUMENT_ADMIN_VIEW_LABEL',
				'type' => 'adminviewfolderlist',
				'title' => false,
				'list' => 'help_documents',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'CHAR(255)',
					'default' => '',
					'GUID' => 'd7f97213-2aa1-457c-9e60-d691cfcd905f',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 2,
					'table' => '',
					'component' => '',
					'entity' => '',
					'value' => '',
					'key' => '',
				],
			],
			'site_view' => [
				'name' => 'site_view',
				'label' => 'COM_COMPONENTBUILDER_HELP_DOCUMENT_SITE_VIEW_LABEL',
				'type' => 'siteviewfolderlist',
				'title' => false,
				'list' => 'help_documents',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'CHAR(255)',
					'default' => '',
					'GUID' => 'd52f96cd-df76-494b-8c25-146bb434b6af',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => [
					'type' => 2,
					'table' => '',
					'component' => '',
					'entity' => '',
					'value' => '',
					'key' => '',
				],
			],
			'alias' => [
				'name' => 'alias',
				'label' => 'COM_COMPONENTBUILDER_HELP_DOCUMENT_ALIAS_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'help_documents',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'CHAR(64)',
					'default' => '',
					'GUID' => '335866ce-b81b-4329-901d-c20254135c9c',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'content' => [
				'name' => 'content',
				'label' => 'COM_COMPONENTBUILDER_HELP_DOCUMENT_CONTENT_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'help_documents',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '8b9665d1-f1d9-4d7e-bc2c-72479bf4686f',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'article' => [
				'name' => 'article',
				'label' => 'COM_COMPONENTBUILDER_HELP_DOCUMENT_ARTICLE_LABEL',
				'type' => 'articles',
				'title' => false,
				'list' => 'help_documents',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => 'f312a8f7-34b9-4731-af00-3ff2092c65b9',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__content',
					'component' => 'com_content',
					'entity' => 'article',
					'value' => 'title',
					'key' => 'id',
				],
			],
			'url' => [
				'name' => 'url',
				'label' => 'COM_COMPONENTBUILDER_HELP_DOCUMENT_URL_LABEL',
				'type' => 'url',
				'title' => false,
				'list' => 'help_documents',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => 'd29d6854-bb4a-4fa2-83a1-5a1b9c9a0eaf',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'target' => [
				'name' => 'target',
				'label' => 'COM_COMPONENTBUILDER_HELP_DOCUMENT_TARGET_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'help_documents',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '5810dc57-35ac-436e-88ff-117e1f501e67',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
		],
		'admin_fields' => [
			'admin_view' => [
				'name' => 'admin_view',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_FIELDS_ADMIN_VIEW_LABEL',
				'type' => 'adminviewsreadonly',
				'title' => true,
				'list' => 'admins_fields',
				'store' => NULL,
				'tab_name' => 'Fields',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '3ac1327d-75d6-4a16-913d-ef4fdf54c11c',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_admin_view',
					'component' => 'com_componentbuilder',
					'entity' => 'admin_view',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addfields' => [
				'name' => 'addfields',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_FIELDS_ADDFIELDS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'admins_fields',
				'store' => 'json',
				'tab_name' => 'Fields',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e64dde45-56cf-4493-927d-830dba2a1f1c',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'field' => [
						'name' => 'field',
						'type' => 'fields',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_field',
							'component' => 'com_componentbuilder',
							'entity' => 'field',
							'value' => 'name',
							'key' => 'guid',
						],
					],
					'list' => [
						'name' => 'list',
						'type' => 'list',
						'link' => NULL,
					],
					'order_list' => [
						'name' => 'order_list',
						'type' => 'integer',
						'link' => NULL,
					],
					'title' => [
						'name' => 'title',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'alias' => [
						'name' => 'alias',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'sort' => [
						'name' => 'sort',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'search' => [
						'name' => 'search',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'filter' => [
						'name' => 'filter',
						'type' => 'list',
						'link' => NULL,
					],
					'link' => [
						'name' => 'link',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'tab' => [
						'name' => 'tab',
						'type' => 'viewtabs',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_admin_view',
							'component' => 'com_componentbuilder',
							'entity' => 'admin_view',
							'value' => 'addtabs',
							'key' => 'id',
						],
					],
					'alignment' => [
						'name' => 'alignment',
						'type' => 'list',
						'link' => NULL,
					],
					'order_edit' => [
						'name' => 'order_edit',
						'type' => 'integer',
						'link' => NULL,
					],
					'permission' => [
						'name' => 'permission',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'admin_fields_conditions' => [
			'admin_view' => [
				'name' => 'admin_view',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_FIELDS_CONDITIONS_ADMIN_VIEW_LABEL',
				'type' => 'adminviewsreadonly',
				'title' => true,
				'list' => 'admins_fields_conditions',
				'store' => NULL,
				'tab_name' => 'Conditions',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '3ac1327d-75d6-4a16-913d-ef4fdf54c11c',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_admin_view',
					'component' => 'com_componentbuilder',
					'entity' => 'admin_view',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addconditions' => [
				'name' => 'addconditions',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_FIELDS_CONDITIONS_ADDCONDITIONS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'admins_fields_conditions',
				'store' => 'json',
				'tab_name' => 'Conditions',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '434a8099-51f0-49d1-bbbb-ed7aa5145a3c',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'target_field' => [
						'name' => 'target_field',
						'type' => 'targetfields',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_field',
							'component' => 'com_componentbuilder',
							'entity' => 'field',
							'value' => 'name',
							'key' => 'guid',
						],
					],
					'target_behavior' => [
						'name' => 'target_behavior',
						'type' => 'list',
						'link' => NULL,
					],
					'target_relation' => [
						'name' => 'target_relation',
						'type' => 'list',
						'link' => NULL,
					],
					'match_field' => [
						'name' => 'match_field',
						'type' => 'matchfield',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_field',
							'component' => 'com_componentbuilder',
							'entity' => 'field',
							'value' => 'name',
							'key' => 'guid',
						],
					],
					'match_behavior' => [
						'name' => 'match_behavior',
						'type' => 'list',
						'link' => NULL,
					],
					'match_options' => [
						'name' => 'match_options',
						'type' => 'textarea',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'admin_fields_relations' => [
			'admin_view' => [
				'name' => 'admin_view',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_FIELDS_RELATIONS_ADMIN_VIEW_LABEL',
				'type' => 'adminviewsreadonly',
				'title' => true,
				'list' => 'admins_fields_relations',
				'store' => NULL,
				'tab_name' => 'Relations',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '3ac1327d-75d6-4a16-913d-ef4fdf54c11c',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_admin_view',
					'component' => 'com_componentbuilder',
					'entity' => 'admin_view',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addrelations' => [
				'name' => 'addrelations',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_FIELDS_RELATIONS_ADDRELATIONS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'admins_fields_relations',
				'store' => 'json',
				'tab_name' => 'Relations',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '4e87b302-e17f-468e-8934-e5aaf78d6765',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'listfield' => [
						'name' => 'listfield',
						'type' => 'listfields',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_field',
							'component' => 'com_componentbuilder',
							'entity' => 'field',
							'value' => 'name',
							'key' => 'guid',
						],
					],
					'column_name' => [
						'name' => 'column_name',
						'type' => 'text',
						'link' => NULL,
					],
					'joinfields' => [
						'name' => 'joinfields',
						'type' => 'joinfields',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_field',
							'component' => 'com_componentbuilder',
							'entity' => 'field',
							'value' => 'name',
							'key' => 'guid',
						],
					],
					'area' => [
						'name' => 'area',
						'type' => 'list',
						'link' => NULL,
					],
					'join_type' => [
						'name' => 'join_type',
						'type' => 'list',
						'link' => NULL,
					],
					'set' => [
						'name' => 'set',
						'type' => 'textarea',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'admin_custom_tabs' => [
			'admin_view' => [
				'name' => 'admin_view',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_CUSTOM_TABS_ADMIN_VIEW_LABEL',
				'type' => 'adminviewsreadonly',
				'title' => true,
				'list' => 'admins_custom_tabs',
				'store' => NULL,
				'tab_name' => 'Tabs',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '3ac1327d-75d6-4a16-913d-ef4fdf54c11c',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_admin_view',
					'component' => 'com_componentbuilder',
					'entity' => 'admin_view',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'tabs' => [
				'name' => 'tabs',
				'label' => 'COM_COMPONENTBUILDER_ADMIN_CUSTOM_TABS_TABS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'admins_custom_tabs',
				'store' => 'json',
				'tab_name' => 'Tabs',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '4b794f1a-1be4-4804-92e6-bda624f444b4',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'tab' => [
						'name' => 'tab',
						'type' => 'viewtabs',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_admin_view',
							'component' => 'com_componentbuilder',
							'entity' => 'admin_view',
							'value' => 'addtabs',
							'key' => 'id',
						],
					],
					'position' => [
						'name' => 'position',
						'type' => 'list',
						'link' => NULL,
					],
					'name' => [
						'name' => 'name',
						'type' => 'text',
						'link' => NULL,
					],
					'permission' => [
						'name' => 'permission',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'html' => [
						'name' => 'html',
						'type' => 'textarea',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'component_admin_views' => [
			'joomla_component' => [
				'name' => 'joomla_component',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_ADMIN_VIEWS_JOOMLA_COMPONENT_LABEL',
				'type' => 'joomlacomponents',
				'title' => true,
				'list' => 'components_admin_views',
				'store' => NULL,
				'tab_name' => 'Views',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '591df1a2-b685-4378-845a-f8a1d1985548',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addadmin_views' => [
				'name' => 'addadmin_views',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_ADMIN_VIEWS_ADDADMIN_VIEWS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_admin_views',
				'store' => 'json',
				'tab_name' => 'Views',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '7e40f307-77e9-407b-8a0a-26dece968d20',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'adminview' => [
						'name' => 'adminview',
						'type' => 'adminviews',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_admin_view',
							'component' => 'com_componentbuilder',
							'entity' => 'admin_view',
							'value' => 'system_name',
							'key' => 'guid',
						],
					],
					'icomoon' => [
						'name' => 'icomoon',
						'type' => 'list',
						'link' => NULL,
					],
					'mainmenu' => [
						'name' => 'mainmenu',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'dashboard_add' => [
						'name' => 'dashboard_add',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'dashboard_list' => [
						'name' => 'dashboard_list',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'submenu' => [
						'name' => 'submenu',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'checkin' => [
						'name' => 'checkin',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'history' => [
						'name' => 'history',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'joomla_fields' => [
						'name' => 'joomla_fields',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'metadata' => [
						'name' => 'metadata',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'access' => [
						'name' => 'access',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'port' => [
						'name' => 'port',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'add_api' => [
						'name' => 'add_api',
						'type' => 'list',
						'link' => NULL,
					],
					'filter' => [
						'name' => 'filter',
						'type' => 'list',
						'link' => NULL,
					],
					'edit_create_site_view' => [
						'name' => 'edit_create_site_view',
						'type' => 'list',
						'link' => NULL,
					],
					'order' => [
						'name' => 'order',
						'type' => 'integer',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'component_site_views' => [
			'joomla_component' => [
				'name' => 'joomla_component',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_SITE_VIEWS_JOOMLA_COMPONENT_LABEL',
				'type' => 'joomlacomponents',
				'title' => true,
				'list' => 'components_site_views',
				'store' => NULL,
				'tab_name' => 'Views',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '591df1a2-b685-4378-845a-f8a1d1985548',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addsite_views' => [
				'name' => 'addsite_views',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_SITE_VIEWS_ADDSITE_VIEWS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_site_views',
				'store' => 'json',
				'tab_name' => 'Views',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '019c6ed4-b16f-452c-8db5-288f9c35a171',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'siteview' => [
						'name' => 'siteview',
						'type' => 'siteviews',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_site_view',
							'component' => 'com_componentbuilder',
							'entity' => 'site_view',
							'value' => 'system_name',
							'key' => 'guid',
						],
					],
					'menu' => [
						'name' => 'menu',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'metadata' => [
						'name' => 'metadata',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'default_view' => [
						'name' => 'default_view',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'access' => [
						'name' => 'access',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'public_access' => [
						'name' => 'public_access',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'component_custom_admin_views' => [
			'joomla_component' => [
				'name' => 'joomla_component',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_CUSTOM_ADMIN_VIEWS_JOOMLA_COMPONENT_LABEL',
				'type' => 'joomlacomponents',
				'title' => true,
				'list' => 'components_custom_admin_views',
				'store' => NULL,
				'tab_name' => 'Views',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '591df1a2-b685-4378-845a-f8a1d1985548',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addcustom_admin_views' => [
				'name' => 'addcustom_admin_views',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_CUSTOM_ADMIN_VIEWS_ADDCUSTOM_ADMIN_VIEWS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_custom_admin_views',
				'store' => 'json',
				'tab_name' => 'Views',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'd0258bfb-9a5e-4edc-94a9-67f52f137d77',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'customadminview' => [
						'name' => 'customadminview',
						'type' => 'customadminviews',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_custom_admin_view',
							'component' => 'com_componentbuilder',
							'entity' => 'custom_admin_view',
							'value' => 'system_name',
							'key' => 'guid',
						],
					],
					'icomoon' => [
						'name' => 'icomoon',
						'type' => 'list',
						'link' => NULL,
					],
					'mainmenu' => [
						'name' => 'mainmenu',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'dashboard_list' => [
						'name' => 'dashboard_list',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'submenu' => [
						'name' => 'submenu',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'metadata' => [
						'name' => 'metadata',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'access' => [
						'name' => 'access',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'adminviews' => [
						'name' => 'adminviews',
						'type' => 'adminsviews',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_admin_view',
							'component' => 'com_componentbuilder',
							'entity' => 'admin_view',
							'value' => 'system_name',
							'key' => 'guid',
						],
					],
					'before' => [
						'name' => 'before',
						'type' => 'componentadminmenus',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_admin_view',
							'component' => 'com_componentbuilder',
							'entity' => 'admin_view',
							'value' => 'system_name',
							'key' => 'guid',
						],
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'component_updates' => [
			'joomla_component' => [
				'name' => 'joomla_component',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_UPDATES_JOOMLA_COMPONENT_LABEL',
				'type' => 'joomlacomponents',
				'title' => true,
				'list' => 'components_updates',
				'store' => NULL,
				'tab_name' => 'Updates',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '591df1a2-b685-4378-845a-f8a1d1985548',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'version_update' => [
				'name' => 'version_update',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_UPDATES_VERSION_UPDATE_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_updates',
				'store' => 'json',
				'tab_name' => 'Updates',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '1b98fc0a-d58f-4c81-a53f-30106e0c30eb',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'version' => [
						'name' => 'version',
						'type' => 'text',
						'link' => NULL,
					],
					'mysql' => [
						'name' => 'mysql',
						'type' => 'textarea',
						'link' => NULL,
					],
					'url' => [
						'name' => 'url',
						'type' => 'url',
						'link' => NULL,
					],
					'update_server_adv' => [
						'name' => 'update_server_adv',
						'type' => 'radio',
						'link' => NULL,
					],
					'update_server_type' => [
						'name' => 'update_server_type',
						'type' => 'list',
						'link' => NULL,
					],
					'update_element' => [
						'name' => 'update_element',
						'type' => 'text',
						'link' => NULL,
					],
					'update_state' => [
						'name' => 'update_state',
						'type' => 'text',
						'link' => NULL,
					],
					'update_target_version' => [
						'name' => 'update_target_version',
						'type' => 'text',
						'link' => NULL,
					],
					'change_log' => [
						'name' => 'change_log',
						'type' => 'textarea',
						'link' => NULL,
					],
					'update_client' => [
						'name' => 'update_client',
						'type' => 'text',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'component_mysql_tweaks' => [
			'joomla_component' => [
				'name' => 'joomla_component',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_MYSQL_TWEAKS_JOOMLA_COMPONENT_LABEL',
				'type' => 'joomlacomponents',
				'title' => true,
				'list' => 'components_mysql_tweaks',
				'store' => NULL,
				'tab_name' => 'Mysql Tweaks',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '591df1a2-b685-4378-845a-f8a1d1985548',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'sql_tweak' => [
				'name' => 'sql_tweak',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_MYSQL_TWEAKS_SQL_TWEAK_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_mysql_tweaks',
				'store' => 'json',
				'tab_name' => 'Mysql Tweaks',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'ce203691-6577-4c0d-90fb-539d07ea76c3',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'adminview' => [
						'name' => 'adminview',
						'type' => 'componentadminviews',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_admin_view',
							'component' => 'com_componentbuilder',
							'entity' => 'admin_view',
							'value' => 'system_name',
							'key' => 'guid',
						],
					],
					'add_sql' => [
						'name' => 'add_sql',
						'type' => 'list',
						'link' => NULL,
					],
					'add_sql_options' => [
						'name' => 'add_sql_options',
						'type' => 'list',
						'link' => NULL,
					],
					'ids' => [
						'name' => 'ids',
						'type' => 'textarea',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'component_custom_admin_menus' => [
			'joomla_component' => [
				'name' => 'joomla_component',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_CUSTOM_ADMIN_MENUS_JOOMLA_COMPONENT_LABEL',
				'type' => 'joomlacomponents',
				'title' => true,
				'list' => 'components_custom_admin_menus',
				'store' => NULL,
				'tab_name' => 'Menus',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '591df1a2-b685-4378-845a-f8a1d1985548',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addcustommenus' => [
				'name' => 'addcustommenus',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_CUSTOM_ADMIN_MENUS_ADDCUSTOMMENUS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_custom_admin_menus',
				'store' => 'json',
				'tab_name' => 'Menus',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '60d68340-14c1-45a9-8240-30f6bdf96345',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'name' => [
						'name' => 'name',
						'type' => 'text',
						'link' => NULL,
					],
					'name_code' => [
						'name' => 'name_code',
						'type' => 'text',
						'link' => NULL,
					],
					'link' => [
						'name' => 'link',
						'type' => 'text',
						'link' => NULL,
					],
					'icon' => [
						'name' => 'icon',
						'type' => 'filelist',
						'link' => NULL,
					],
					'mainmenu' => [
						'name' => 'mainmenu',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'dashboard_list' => [
						'name' => 'dashboard_list',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'submenu' => [
						'name' => 'submenu',
						'type' => 'checkbox',
						'link' => NULL,
					],
					'before' => [
						'name' => 'before',
						'type' => 'componentadminmenus',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_admin_view',
							'component' => 'com_componentbuilder',
							'entity' => 'admin_view',
							'value' => 'system_name',
							'key' => 'guid',
						],
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'component_router' => [
			'joomla_component' => [
				'name' => 'joomla_component',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_ROUTER_JOOMLA_COMPONENT_LABEL',
				'type' => 'joomlacomponents',
				'title' => true,
				'list' => 'components_routers',
				'store' => NULL,
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '591df1a2-b685-4378-845a-f8a1d1985548',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'mode_constructor_before_parent' => [
				'name' => 'mode_constructor_before_parent',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_ROUTER_MODE_CONSTRUCTOR_BEFORE_PARENT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'components_routers',
				'store' => NULL,
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '4582a192-d46e-4dd9-9bbd-ee2e2ddd20e5',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'mode_constructor_after_parent' => [
				'name' => 'mode_constructor_after_parent',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_ROUTER_MODE_CONSTRUCTOR_AFTER_PARENT_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'components_routers',
				'store' => NULL,
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '2b31ca2e-11ff-4f57-8c90-2b9bed743202',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'mode_methods' => [
				'name' => 'mode_methods',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_ROUTER_MODE_METHODS_LABEL',
				'type' => 'radio',
				'title' => false,
				'list' => 'components_routers',
				'store' => NULL,
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'TINYINT(1)',
					'default' => '0',
					'GUID' => '25fead1f-bb12-40e6-9e03-4e31b236f2e1',
					'null_switch' => 'NOT NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'methods_code' => [
				'name' => 'methods_code',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_ROUTER_METHODS_CODE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'components_routers',
				'store' => 'base64',
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '3ab69198-0827-452b-93a4-6c7bb02ae7ae',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'constructor_after_parent_code' => [
				'name' => 'constructor_after_parent_code',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_ROUTER_CONSTRUCTOR_AFTER_PARENT_CODE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'components_routers',
				'store' => 'base64',
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'a411a365-69c8-4f70-a734-25956a9f3393',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'constructor_before_parent_manual' => [
				'name' => 'constructor_before_parent_manual',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_ROUTER_CONSTRUCTOR_BEFORE_PARENT_MANUAL_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_routers',
				'store' => 'json',
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '2321b486-80c4-4e24-81c5-6ddd205592ac',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'siteview' => [
						'name' => 'siteview',
						'type' => 'siteviews',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_site_view',
							'component' => 'com_componentbuilder',
							'entity' => 'site_view',
							'value' => 'system_name',
							'key' => 'guid',
						],
					],
				],
			],
			'constructor_before_parent_code' => [
				'name' => 'constructor_before_parent_code',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_ROUTER_CONSTRUCTOR_BEFORE_PARENT_CODE_LABEL',
				'type' => 'editor',
				'title' => false,
				'list' => 'components_routers',
				'store' => 'base64',
				'tab_name' => 'Settings',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '65544263-3215-4574-a977-dbf8ac2ce72a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'component_config' => [
			'joomla_component' => [
				'name' => 'joomla_component',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_CONFIG_JOOMLA_COMPONENT_LABEL',
				'type' => 'joomlacomponents',
				'title' => true,
				'list' => 'components_config',
				'store' => NULL,
				'tab_name' => 'Options',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '591df1a2-b685-4378-845a-f8a1d1985548',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addconfig' => [
				'name' => 'addconfig',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_CONFIG_ADDCONFIG_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_config',
				'store' => 'json',
				'tab_name' => 'Options',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'bd3d6119-a5f6-4c0f-b1bd-b1e243dac32b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'field' => [
						'name' => 'field',
						'type' => 'fields',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_field',
							'component' => 'com_componentbuilder',
							'entity' => 'field',
							'value' => 'name',
							'key' => 'guid',
						],
					],
					'custom_value' => [
						'name' => 'custom_value',
						'type' => 'textarea',
						'link' => NULL,
					],
					'tabname' => [
						'name' => 'tabname',
						'type' => 'text',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'component_dashboard' => [
			'joomla_component' => [
				'name' => 'joomla_component',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_DASHBOARD_JOOMLA_COMPONENT_LABEL',
				'type' => 'joomlacomponents',
				'title' => true,
				'list' => 'components_dashboard',
				'store' => NULL,
				'tab_name' => 'Dashboard',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '591df1a2-b685-4378-845a-f8a1d1985548',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'dashboard_tab' => [
				'name' => 'dashboard_tab',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_DASHBOARD_DASHBOARD_TAB_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_dashboard',
				'store' => 'json',
				'tab_name' => 'Dashboard',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'fd0cceef-975e-4daa-a529-2c98446b9df3',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'name' => [
						'name' => 'name',
						'type' => 'text',
						'link' => NULL,
					],
					'header' => [
						'name' => 'header',
						'type' => 'text',
						'link' => NULL,
					],
					'html' => [
						'name' => 'html',
						'type' => 'textarea',
						'link' => NULL,
					],
				],
			],
			'php_dashboard_methods' => [
				'name' => 'php_dashboard_methods',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_DASHBOARD_PHP_DASHBOARD_METHODS_LABEL',
				'type' => 'textarea',
				'title' => false,
				'list' => 'components_dashboard',
				'store' => 'base64',
				'tab_name' => 'Dashboard',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => 'e6c421c3-2f37-429a-a8d9-b552bed392a5',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'component_files_folders' => [
			'joomla_component' => [
				'name' => 'joomla_component',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_FILES_FOLDERS_JOOMLA_COMPONENT_LABEL',
				'type' => 'joomlacomponents',
				'title' => true,
				'list' => 'components_files_folders',
				'store' => NULL,
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '591df1a2-b685-4378-845a-f8a1d1985548',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addfoldersfullpath' => [
				'name' => 'addfoldersfullpath',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_FILES_FOLDERS_ADDFOLDERSFULLPATH_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_files_folders',
				'store' => 'json',
				'tab_name' => 'Advance',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'fe780ac8-ed30-481c-8700-84472caa0072',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'folderpath' => [
						'name' => 'folderpath',
						'type' => 'text',
						'link' => NULL,
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'rename' => [
						'name' => 'rename',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addfilesfullpath' => [
				'name' => 'addfilesfullpath',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_FILES_FOLDERS_ADDFILESFULLPATH_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_files_folders',
				'store' => 'json',
				'tab_name' => 'Advance',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'e5f50226-6749-4bd3-9aa1-cf3db6ce448d',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'filepath' => [
						'name' => 'filepath',
						'type' => 'text',
						'link' => NULL,
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'notnew' => [
						'name' => 'notnew',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addfolders' => [
				'name' => 'addfolders',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_FILES_FOLDERS_ADDFOLDERS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_files_folders',
				'store' => 'json',
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '483f2c40-2348-4168-817b-471f5ba96984',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'folder' => [
						'name' => 'folder',
						'type' => 'customfolderlist',
						'link' => [
							'type' => 2,
							'table' => '',
							'component' => '',
							'entity' => '',
							'value' => '',
							'key' => '',
						],
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'rename' => [
						'name' => 'rename',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addfiles' => [
				'name' => 'addfiles',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_FILES_FOLDERS_ADDFILES_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_files_folders',
				'store' => 'json',
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '143f8b3d-5b58-4b26-aaad-a5f14a57865b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'file' => [
						'name' => 'file',
						'type' => 'customfilelist',
						'link' => [
							'type' => 2,
							'table' => '',
							'component' => '',
							'entity' => '',
							'value' => '',
							'key' => '',
						],
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'notnew' => [
						'name' => 'notnew',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'component_placeholders' => [
			'joomla_component' => [
				'name' => 'joomla_component',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_PLACEHOLDERS_JOOMLA_COMPONENT_LABEL',
				'type' => 'joomlacomponents',
				'title' => true,
				'list' => 'components_placeholders',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '591df1a2-b685-4378-845a-f8a1d1985548',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addplaceholders' => [
				'name' => 'addplaceholders',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_PLACEHOLDERS_ADDPLACEHOLDERS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_placeholders',
				'store' => 'json',
				'tab_name' => 'Details',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'fb4e322b-46c0-47e5-b9eb-70c39456d80d',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'target' => [
						'name' => 'target',
						'type' => 'text',
						'link' => NULL,
					],
					'value' => [
						'name' => 'value',
						'type' => 'textarea',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'component_plugins' => [
			'joomla_component' => [
				'name' => 'joomla_component',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_PLUGINS_JOOMLA_COMPONENT_LABEL',
				'type' => 'joomlacomponents',
				'title' => true,
				'list' => 'components_plugins',
				'store' => NULL,
				'tab_name' => 'Plugins',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '591df1a2-b685-4378-845a-f8a1d1985548',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addjoomla_plugins' => [
				'name' => 'addjoomla_plugins',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_PLUGINS_ADDJOOMLA_PLUGINS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_plugins',
				'store' => 'json',
				'tab_name' => 'Plugins',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '9c787541-ed54-4516-bc21-9b6fe2e2e516',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'plugin' => [
						'name' => 'plugin',
						'type' => 'joomlaplugins',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_joomla_plugin',
							'component' => 'com_componentbuilder',
							'entity' => 'joomla_plugin',
							'value' => 'system_name',
							'key' => 'guid',
						],
					],
					'target' => [
						'name' => 'target',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'component_modules' => [
			'joomla_component' => [
				'name' => 'joomla_component',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_MODULES_JOOMLA_COMPONENT_LABEL',
				'type' => 'joomlacomponents',
				'title' => true,
				'list' => 'components_modules',
				'store' => NULL,
				'tab_name' => 'Modules',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '591df1a2-b685-4378-845a-f8a1d1985548',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_component',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_component',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addjoomla_modules' => [
				'name' => 'addjoomla_modules',
				'label' => 'COM_COMPONENTBUILDER_COMPONENT_MODULES_ADDJOOMLA_MODULES_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'components_modules',
				'store' => 'json',
				'tab_name' => 'Modules',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '5985271b-0795-4204-8739-ffa3d898bae8',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'module' => [
						'name' => 'module',
						'type' => 'joomlamodules',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_joomla_module',
							'component' => 'com_componentbuilder',
							'entity' => 'joomla_module',
							'value' => 'system_name',
							'key' => 'guid',
						],
					],
					'target' => [
						'name' => 'target',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'snippet_type' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_TYPE_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'snippet_types',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'description' => [
				'name' => 'description',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_TYPE_DESCRIPTION_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'snippet_types',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '09b25ca3-6c9f-4b6d-ad80-e0989c221289',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_SNIPPET_TYPE_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'snippet_types',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'library_config' => [
			'library' => [
				'name' => 'library',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_CONFIG_LIBRARY_LABEL',
				'type' => 'libraryreadonly',
				'title' => true,
				'list' => 'libraries_config',
				'store' => NULL,
				'tab_name' => 'Tweaks',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'c6dddec0-c373-4069-bdcc-66acd474b648',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_library',
					'component' => 'com_componentbuilder',
					'entity' => 'library',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'addconfig' => [
				'name' => 'addconfig',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_CONFIG_ADDCONFIG_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'libraries_config',
				'store' => 'json',
				'tab_name' => 'Tweaks',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'bd3d6119-a5f6-4c0f-b1bd-b1e243dac32b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'field' => [
						'name' => 'field',
						'type' => 'fields',
						'link' => [
							'type' => 1,
							'table' => '#__componentbuilder_field',
							'component' => 'com_componentbuilder',
							'entity' => 'field',
							'value' => 'name',
							'key' => 'guid',
						],
					],
					'custom_value' => [
						'name' => 'custom_value',
						'type' => 'textarea',
						'link' => NULL,
					],
					'tabname' => [
						'name' => 'tabname',
						'type' => 'text',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'library_files_folders_urls' => [
			'library' => [
				'name' => 'library',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_FILES_FOLDERS_URLS_LIBRARY_LABEL',
				'type' => 'libraryreadonly',
				'title' => true,
				'list' => 'libraries_files_folders_urls',
				'store' => NULL,
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'c6dddec0-c373-4069-bdcc-66acd474b648',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_library',
					'component' => 'com_componentbuilder',
					'entity' => 'library',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'addfoldersfullpath' => [
				'name' => 'addfoldersfullpath',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_FILES_FOLDERS_URLS_ADDFOLDERSFULLPATH_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'libraries_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Advance',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'fe780ac8-ed30-481c-8700-84472caa0072',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'folderpath' => [
						'name' => 'folderpath',
						'type' => 'text',
						'link' => NULL,
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'rename' => [
						'name' => 'rename',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addfilesfullpath' => [
				'name' => 'addfilesfullpath',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_FILES_FOLDERS_URLS_ADDFILESFULLPATH_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'libraries_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Advance',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'e5f50226-6749-4bd3-9aa1-cf3db6ce448d',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'filepath' => [
						'name' => 'filepath',
						'type' => 'text',
						'link' => NULL,
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'notnew' => [
						'name' => 'notnew',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addfolders' => [
				'name' => 'addfolders',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_FILES_FOLDERS_URLS_ADDFOLDERS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'libraries_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '483f2c40-2348-4168-817b-471f5ba96984',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'folder' => [
						'name' => 'folder',
						'type' => 'customfolderlist',
						'link' => [
							'type' => 2,
							'table' => '',
							'component' => '',
							'entity' => '',
							'value' => '',
							'key' => '',
						],
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'rename' => [
						'name' => 'rename',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addfiles' => [
				'name' => 'addfiles',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_FILES_FOLDERS_URLS_ADDFILES_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'libraries_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '143f8b3d-5b58-4b26-aaad-a5f14a57865b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'file' => [
						'name' => 'file',
						'type' => 'customfilelist',
						'link' => [
							'type' => 2,
							'table' => '',
							'component' => '',
							'entity' => '',
							'value' => '',
							'key' => '',
						],
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'notnew' => [
						'name' => 'notnew',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addurls' => [
				'name' => 'addurls',
				'label' => 'COM_COMPONENTBUILDER_LIBRARY_FILES_FOLDERS_URLS_ADDURLS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'libraries_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'c3474b9a-4677-4574-bda3-a4865569ac18',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'url' => [
						'name' => 'url',
						'type' => 'url',
						'link' => NULL,
					],
					'type' => [
						'name' => 'type',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'joomla_module_updates' => [
			'joomla_module' => [
				'name' => 'joomla_module',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_UPDATES_JOOMLA_MODULE_LABEL',
				'type' => 'joomlamodules',
				'title' => true,
				'list' => 'joomla_modules_updates',
				'store' => NULL,
				'tab_name' => 'Updates',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '79251a06-b217-4b3a-8485-b7286cb15bf6',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_module',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_module',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'version_update' => [
				'name' => 'version_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_UPDATES_VERSION_UPDATE_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_modules_updates',
				'store' => 'json',
				'tab_name' => 'Updates',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '7d49445c-4e95-4fd5-a2a4-a20bbcd4d812',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'version' => [
						'name' => 'version',
						'type' => 'text',
						'link' => NULL,
					],
					'mysql' => [
						'name' => 'mysql',
						'type' => 'textarea',
						'link' => NULL,
					],
					'url' => [
						'name' => 'url',
						'type' => 'url',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'joomla_module_files_folders_urls' => [
			'joomla_module' => [
				'name' => 'joomla_module',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_FILES_FOLDERS_URLS_JOOMLA_MODULE_LABEL',
				'type' => 'joomlamodules',
				'title' => true,
				'list' => 'joomla_modules_files_folders_urls',
				'store' => NULL,
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '79251a06-b217-4b3a-8485-b7286cb15bf6',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_module',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_module',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addfoldersfullpath' => [
				'name' => 'addfoldersfullpath',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_FILES_FOLDERS_URLS_ADDFOLDERSFULLPATH_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_modules_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Advance',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'fe780ac8-ed30-481c-8700-84472caa0072',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'folderpath' => [
						'name' => 'folderpath',
						'type' => 'text',
						'link' => NULL,
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'rename' => [
						'name' => 'rename',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addfilesfullpath' => [
				'name' => 'addfilesfullpath',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_FILES_FOLDERS_URLS_ADDFILESFULLPATH_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_modules_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Advance',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'e5f50226-6749-4bd3-9aa1-cf3db6ce448d',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'filepath' => [
						'name' => 'filepath',
						'type' => 'text',
						'link' => NULL,
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'notnew' => [
						'name' => 'notnew',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addfolders' => [
				'name' => 'addfolders',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_FILES_FOLDERS_URLS_ADDFOLDERS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_modules_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '483f2c40-2348-4168-817b-471f5ba96984',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'folder' => [
						'name' => 'folder',
						'type' => 'customfolderlist',
						'link' => [
							'type' => 2,
							'table' => '',
							'component' => '',
							'entity' => '',
							'value' => '',
							'key' => '',
						],
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'rename' => [
						'name' => 'rename',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addfiles' => [
				'name' => 'addfiles',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_FILES_FOLDERS_URLS_ADDFILES_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_modules_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '143f8b3d-5b58-4b26-aaad-a5f14a57865b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'file' => [
						'name' => 'file',
						'type' => 'customfilelist',
						'link' => [
							'type' => 2,
							'table' => '',
							'component' => '',
							'entity' => '',
							'value' => '',
							'key' => '',
						],
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'notnew' => [
						'name' => 'notnew',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addurls' => [
				'name' => 'addurls',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_MODULE_FILES_FOLDERS_URLS_ADDURLS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_modules_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'c3474b9a-4677-4574-bda3-a4865569ac18',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'url' => [
						'name' => 'url',
						'type' => 'url',
						'link' => NULL,
					],
					'type' => [
						'name' => 'type',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'joomla_plugin_group' => [
			'name' => [
				'name' => 'name',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_GROUP_NAME_LABEL',
				'type' => 'text',
				'title' => true,
				'list' => 'joomla_plugin_groups',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(255)',
					'default' => '',
					'GUID' => '5d3d34dd-4876-4c6a-86ab-b4e162f22c08',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'class_extends' => [
				'name' => 'class_extends',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_GROUP_CLASS_EXTENDS_LABEL',
				'type' => 'classextends',
				'title' => false,
				'list' => 'joomla_plugin_groups',
				'store' => NULL,
				'tab_name' => 'Details',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'b1edf3cc-3473-41c7-9825-372333299846',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_class_extends',
					'component' => 'com_componentbuilder',
					'entity' => 'class_extends',
					'value' => 'name',
					'key' => 'guid',
				],
			],
			'guid' => [
				'name' => 'guid',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_GROUP_GUID_LABEL',
				'type' => 'text',
				'title' => false,
				'list' => 'joomla_plugin_groups',
				'store' => NULL,
				'tab_name' => 'publishing',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => '5aa57bbe-7b19-4db9-915c-561863458d2b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => NULL,
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'joomla_plugin_updates' => [
			'joomla_plugin' => [
				'name' => 'joomla_plugin',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_UPDATES_JOOMLA_PLUGIN_LABEL',
				'type' => 'joomlaplugins',
				'title' => true,
				'list' => 'joomla_plugins_updates',
				'store' => NULL,
				'tab_name' => 'Updates',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'd5aeaa99-9fba-4b67-b850-68e19256520a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_plugin',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_plugin',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'version_update' => [
				'name' => 'version_update',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_UPDATES_VERSION_UPDATE_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_plugins_updates',
				'store' => 'json',
				'tab_name' => 'Updates',
				'db' => [
					'type' => 'MEDIUMTEXT',
					'default' => 'EMPTY',
					'GUID' => '7d49445c-4e95-4fd5-a2a4-a20bbcd4d812',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'version' => [
						'name' => 'version',
						'type' => 'text',
						'link' => NULL,
					],
					'mysql' => [
						'name' => 'mysql',
						'type' => 'textarea',
						'link' => NULL,
					],
					'url' => [
						'name' => 'url',
						'type' => 'url',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
		'joomla_plugin_files_folders_urls' => [
			'joomla_plugin' => [
				'name' => 'joomla_plugin',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_FILES_FOLDERS_URLS_JOOMLA_PLUGIN_LABEL',
				'type' => 'joomlaplugins',
				'title' => true,
				'list' => 'joomla_plugins_files_folders_urls',
				'store' => NULL,
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'VARCHAR(36)',
					'default' => '',
					'GUID' => 'd5aeaa99-9fba-4b67-b850-68e19256520a',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => true,
				],
				'link' => [
					'type' => 1,
					'table' => '#__componentbuilder_joomla_plugin',
					'component' => 'com_componentbuilder',
					'entity' => 'joomla_plugin',
					'value' => 'system_name',
					'key' => 'guid',
				],
			],
			'addfoldersfullpath' => [
				'name' => 'addfoldersfullpath',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_FILES_FOLDERS_URLS_ADDFOLDERSFULLPATH_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_plugins_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Advance',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'fe780ac8-ed30-481c-8700-84472caa0072',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'folderpath' => [
						'name' => 'folderpath',
						'type' => 'text',
						'link' => NULL,
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'rename' => [
						'name' => 'rename',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addfilesfullpath' => [
				'name' => 'addfilesfullpath',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_FILES_FOLDERS_URLS_ADDFILESFULLPATH_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_plugins_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Advance',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'e5f50226-6749-4bd3-9aa1-cf3db6ce448d',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'filepath' => [
						'name' => 'filepath',
						'type' => 'text',
						'link' => NULL,
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'notnew' => [
						'name' => 'notnew',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addfolders' => [
				'name' => 'addfolders',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_FILES_FOLDERS_URLS_ADDFOLDERS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_plugins_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '483f2c40-2348-4168-817b-471f5ba96984',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'folder' => [
						'name' => 'folder',
						'type' => 'customfolderlist',
						'link' => [
							'type' => 2,
							'table' => '',
							'component' => '',
							'entity' => '',
							'value' => '',
							'key' => '',
						],
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'rename' => [
						'name' => 'rename',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addfiles' => [
				'name' => 'addfiles',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_FILES_FOLDERS_URLS_ADDFILES_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_plugins_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => '143f8b3d-5b58-4b26-aaad-a5f14a57865b',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'file' => [
						'name' => 'file',
						'type' => 'customfilelist',
						'link' => [
							'type' => 2,
							'table' => '',
							'component' => '',
							'entity' => '',
							'value' => '',
							'key' => '',
						],
					],
					'path' => [
						'name' => 'path',
						'type' => 'text',
						'link' => NULL,
					],
					'notnew' => [
						'name' => 'notnew',
						'type' => 'checkbox',
						'link' => NULL,
					],
				],
			],
			'addurls' => [
				'name' => 'addurls',
				'label' => 'COM_COMPONENTBUILDER_JOOMLA_PLUGIN_FILES_FOLDERS_URLS_ADDURLS_LABEL',
				'type' => 'subform',
				'title' => false,
				'list' => 'joomla_plugins_files_folders_urls',
				'store' => 'json',
				'tab_name' => 'Basic',
				'db' => [
					'type' => 'TEXT',
					'default' => 'EMPTY',
					'GUID' => 'c3474b9a-4677-4574-bda3-a4865569ac18',
					'null_switch' => 'NULL',
					'unique_key' => false,
					'key' => false,
				],
				'link' => NULL,
				'fields' => [
					'url' => [
						'name' => 'url',
						'type' => 'url',
						'link' => NULL,
					],
					'type' => [
						'name' => 'type',
						'type' => 'list',
						'link' => NULL,
					],
				],
			],
			'access' => [
				'name' => 'access',
				'label' => 'Access',
				'type' => 'accesslevel',
				'title' => false,
				'store' => NULL,
				'tab_name' => NULL,
				'db' => [
					'type' => 'INT(10) unsigned',
					'default' => '0',
					'key' => true,
					'null_switch' => 'NULL',
				],
			],
		],
	];
}

